# Copyright 2014 Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ddt
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
import testtools
from testtools import testcase as tc

from manila_tempest_tests.common import constants
from manila_tempest_tests.common import waiters
from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils

CONF = config.CONF
LATEST_MICROVERSION = CONF.share.max_api_microversion


@ddt.ddt
class SharesActionsTest(base.BaseSharesMixedTest):
    """Covers share functionality, that doesn't related to share type."""

    @classmethod
    def resource_setup(cls):
        super(SharesActionsTest, cls).resource_setup()

        cls.shares = []

        # create share_type
        extra_specs = {}
        if CONF.share.capability_snapshot_support:
            extra_specs.update({'snapshot_support': True})
        if CONF.share.capability_create_share_from_snapshot_support:
            extra_specs.update({'create_share_from_snapshot_support': True})
        cls.share_type = cls.create_share_type(extra_specs=extra_specs)
        cls.share_type_id = cls.share_type['id']

        # create share
        cls.share_name = data_utils.rand_name("tempest-share-name")
        cls.share_desc = data_utils.rand_name("tempest-share-description")
        cls.metadata = {
            'foo_key_share_1': 'foo_value_share_1',
            'bar_key_share_1': 'foo_value_share_1',
        }
        cls.shares.append(cls.create_share(
            name=cls.share_name,
            description=cls.share_desc,
            metadata=cls.metadata,
            share_type_id=cls.share_type_id,
        ))

        if CONF.share.run_snapshot_tests:
            # create snapshot
            cls.snap_name = data_utils.rand_name("tempest-snapshot-name")
            cls.snap_desc = data_utils.rand_name(
                "tempest-snapshot-description")
            cls.snap = cls.create_snapshot_wait_for_active(
                cls.shares[0]["id"], cls.snap_name, cls.snap_desc)

            if CONF.share.capability_create_share_from_snapshot_support:

                # create second share from snapshot for purposes of sorting and
                # snapshot filtering
                cls.share_name2 = data_utils.rand_name("tempest-share-name")
                cls.share_desc2 = data_utils.rand_name(
                    "tempest-share-description")
                cls.metadata2 = {
                    'foo_key_share_2': 'foo_value_share_2',
                    'bar_key_share_2': 'foo_value_share_2',
                }
                cls.shares.append(cls.create_share(
                    name=cls.share_name2,
                    description=cls.share_desc2,
                    metadata=cls.metadata2,
                    snapshot_id=cls.snap['id'],
                ))

    def _get_share(self, version):

        # get share
        share = self.shares_v2_client.get_share(
            self.shares[0]['id'], version=str(version))['share']

        # verify keys
        expected_keys = [
            "status", "description", "links", "availability_zone",
            "created_at", "project_id", "volume_type", "share_proto", "name",
            "snapshot_id", "id", "size", "share_network_id", "metadata",
            "snapshot_id", "is_public",
        ]
        if utils.is_microversion_lt(version, '2.9'):
            expected_keys.extend(["export_location", "export_locations"])
        if utils.is_microversion_ge(version, '2.2'):
            expected_keys.append("snapshot_support")
        if utils.is_microversion_ge(version, '2.5'):
            expected_keys.append("share_type_name")
        if utils.is_microversion_ge(version, '2.10'):
            expected_keys.append("access_rules_status")
        if utils.is_microversion_ge(version, '2.11'):
            expected_keys.append("replication_type")
        if utils.is_microversion_ge(version, '2.16'):
            expected_keys.append("user_id")
        if utils.is_microversion_ge(version, '2.24'):
            expected_keys.append("create_share_from_snapshot_support")
        if utils.is_microversion_ge(version,
                                    constants.REVERT_TO_SNAPSHOT_MICROVERSION):
            expected_keys.append("revert_to_snapshot_support")
        actual_keys = list(share.keys())
        [self.assertIn(key, actual_keys) for key in expected_keys]

        # verify values
        msg = "Expected name: '%s', actual name: '%s'" % (self.share_name,
                                                          share["name"])
        self.assertEqual(self.share_name, str(share["name"]), msg)

        msg = ("Expected description: '%s', "
               "actual description: '%s'" % (self.share_desc,
                                             share["description"]))
        self.assertEqual(
            self.share_desc, str(share["description"]), msg)

        msg = "Expected size: '%s', actual size: '%s'" % (
            CONF.share.share_size, share["size"])
        self.assertEqual(CONF.share.share_size, int(share["size"]), msg)

    @decorators.idempotent_id('188badb2-0ca3-44e5-abca-3029475d7f73')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_get_share_v2_1(self):
        self._get_share('2.1')

    @decorators.idempotent_id('45ec8e36-cb8a-4a03-8c41-ba5d5dc2a5c5')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_get_share_with_snapshot_support_key(self):
        self._get_share('2.2')

    @decorators.idempotent_id('b803a076-593a-469d-ab75-7e67ac294dd5')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported('2.6')
    def test_get_share_with_share_type_name_key(self):
        self._get_share('2.6')

    @decorators.idempotent_id('c13c1cf5-c708-4e62-8b71-c47dbb45a7a0')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported('2.9')
    def test_get_share_export_locations_removed(self):
        self._get_share('2.9')

    @decorators.idempotent_id('2c439716-c34e-46f1-b055-41e411fb4e66')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported('2.10')
    def test_get_share_with_access_rules_status(self):
        self._get_share('2.10')

    @decorators.idempotent_id('67c9ad27-f26d-4ff9-a528-a35b4c5a55c1')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported('2.11')
    def test_get_share_with_replication_type_key(self):
        self._get_share('2.11')

    @decorators.idempotent_id('2a61703d-5a62-49c5-b143-fea0c9125cd9')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported('2.16')
    def test_get_share_with_user_id(self):
        self._get_share('2.16')

    @decorators.idempotent_id('2899055e-607d-4423-b1b7-27eefa6ce5f0')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported('2.24')
    def test_get_share_with_create_share_from_snapshot_support(self):
        self._get_share('2.24')

    @decorators.idempotent_id('04c359b5-74b3-4a07-9f9e-f0ac1867d58d')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported(
        constants.REVERT_TO_SNAPSHOT_MICROVERSION)
    def test_get_share_with_revert_to_snapshot_support(self):
        self._get_share(constants.REVERT_TO_SNAPSHOT_MICROVERSION)

    @decorators.idempotent_id('7d61311a-81b0-481c-abb9-cfb0b4f82e29')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_list_shares(self):

        # list shares
        shares = self.shares_v2_client.list_shares()['shares']

        # verify keys
        keys = ["name", "id", "links"]
        [self.assertIn(key, sh.keys()) for sh in shares for key in keys]

        # our share id in list and have no duplicates
        for share in self.shares:
            gen = [sid["id"] for sid in shares if sid["id"] in share["id"]]
            msg = "expected id lists %s times in share list" % (len(gen))
            self.assertEqual(1, len(gen), msg)

    def _list_shares_with_detail(self, version):

        # list shares
        shares = self.shares_v2_client.list_shares_with_detail(
            version=str(version))['shares']

        # verify keys
        keys = [
            "status", "description", "links", "availability_zone",
            "created_at", "project_id", "volume_type", "share_proto", "name",
            "snapshot_id", "id", "size", "share_network_id", "metadata",
            "snapshot_id", "is_public", "share_type",
        ]
        if utils.is_microversion_lt(version, '2.9'):
            keys.extend(["export_location", "export_locations"])
        if utils.is_microversion_ge(version, '2.2'):
            keys.append("snapshot_support")
        if utils.is_microversion_ge(version, '2.6'):
            keys.append("share_type_name")
        if utils.is_microversion_ge(version, '2.10'):
            keys.append("access_rules_status")
        if utils.is_microversion_ge(version, '2.11'):
            keys.append("replication_type")
        if utils.is_microversion_ge(version, '2.16'):
            keys.append("user_id")
        if utils.is_microversion_ge(version, '2.24'):
            keys.append("create_share_from_snapshot_support")
        if utils.is_microversion_ge(version,
                                    constants.REVERT_TO_SNAPSHOT_MICROVERSION):
            keys.append("revert_to_snapshot_support")
        [self.assertIn(key, sh.keys()) for sh in shares for key in keys]

        # our shares in list and have no duplicates
        for share in self.shares:
            gen = [sid["id"] for sid in shares if sid["id"] in share["id"]]
            msg = "expected id lists %s times in share list" % (len(gen))
            self.assertEqual(1, len(gen), msg)

    @decorators.idempotent_id('d88a157a-fe4d-456e-90ba-f0bd9e5d02ec')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_list_shares_with_detail_v2_1(self):
        self._list_shares_with_detail('2.1')

    @decorators.idempotent_id('ee57db25-0fbf-40ce-8dc1-070845d245a7')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_list_shares_with_detail_and_snapshot_support_key(self):
        self._list_shares_with_detail('2.2')

    @decorators.idempotent_id('38a4cf70-8ed9-4a82-a338-9f20463b1147')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported('2.6')
    def test_list_shares_with_detail_share_type_name_key(self):
        self._list_shares_with_detail('2.6')

    @decorators.idempotent_id('a313cd85-53e5-48b7-b4a7-14e9273e843d')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported('2.9')
    def test_list_shares_with_detail_export_locations_removed(self):
        self._list_shares_with_detail('2.9')

    @decorators.idempotent_id('0de4dc84-bace-4b9a-8470-a40c4a9b14b4')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported('2.10')
    def test_list_shares_with_detail_with_access_rules_status(self):
        self._list_shares_with_detail('2.10')

    @decorators.idempotent_id('70097cf3-76b8-49d2-b1c7-ad59394dd978')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported('2.11')
    def test_list_shares_with_detail_replication_type_key(self):
        self._list_shares_with_detail('2.11')

    @decorators.idempotent_id('f8e57190-b105-4937-b96e-55725eb64739')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported('2.16')
    def test_list_shares_with_user_id(self):
        self._list_shares_with_detail('2.16')

    @decorators.idempotent_id('21410324-e0b5-452a-8c7e-599a27267e1d')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_list_shares_with_detail_and_create_share_from_snapshot_support(
            self):
        self._list_shares_with_detail('2.24')

    @decorators.idempotent_id('45041fc9-bb0d-4dd6-ac0d-a40a22a91f3f')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported(
        constants.REVERT_TO_SNAPSHOT_MICROVERSION)
    def test_list_shares_with_detail_with_revert_to_snapshot_support(self):
        self._list_shares_with_detail(
            constants.REVERT_TO_SNAPSHOT_MICROVERSION)

    @decorators.idempotent_id('bc24dc42-050b-4105-a90a-f5649cbd8e49')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_list_shares_with_detail_filter_by_metadata(self):
        filters = {'metadata': self.metadata}

        # list shares
        shares = self.shares_client.list_shares_with_detail(
            params=filters)['shares']

        # verify response
        self.assertGreater(len(shares), 0)
        for share in shares:
            self.assertLessEqual(filters['metadata'].items(),
                                 share['metadata'].items())
        if CONF.share.capability_create_share_from_snapshot_support:
            self.assertFalse(self.shares[1]['id'] in [s['id'] for s in shares])

    @decorators.idempotent_id('685286c7-1df6-48c3-839d-8162737446b8')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @testtools.skipIf(
        not CONF.share.multitenancy_enabled, "Only for multitenancy.")
    def test_list_shares_with_detail_filter_by_share_network_id(self):
        base_share = self.shares_client.get_share(
            self.shares[0]['id'])['share']
        filters = {'share_network_id': base_share['share_network_id']}

        # list shares
        shares = self.shares_client.list_shares_with_detail(
            params=filters)['shares']

        # verify response
        self.assertGreater(len(shares), 0)
        for share in shares:
            self.assertEqual(
                filters['share_network_id'], share['share_network_id'])

    @decorators.idempotent_id('fd87884d-71a4-4ca8-8b2b-07b4df2de3bd')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @testtools.skipUnless(CONF.share.run_snapshot_tests,
                          "Snapshot tests are disabled.")
    @testtools.skipUnless(
        CONF.share.capability_create_share_from_snapshot_support,
        "Create share from snapshot tests are disabled.")
    def test_list_shares_with_detail_filter_by_snapshot_id(self):
        filters = {'snapshot_id': self.snap['id']}

        # list shares
        shares = self.shares_client.list_shares_with_detail(
            params=filters)['shares']

        # verify response
        self.assertGreater(len(shares), 0)
        for share in shares:
            self.assertEqual(filters['snapshot_id'], share['snapshot_id'])
        self.assertFalse(self.shares[0]['id'] in [s['id'] for s in shares])

    @decorators.idempotent_id('70a9d947-bd45-46db-b529-6c48c0ff8985')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_list_shares_with_detail_with_asc_sorting(self):
        filters = {'sort_key': 'created_at', 'sort_dir': 'asc'}

        # list shares
        shares = self.shares_client.list_shares_with_detail(
            params=filters)['shares']

        # verify response
        self.assertGreater(len(shares), 0)
        sorted_list = [share['created_at'] for share in shares]
        self.assertEqual(sorted(sorted_list), sorted_list)

    @decorators.idempotent_id('866194c6-3910-409e-ad81-2cff223cfaaf')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_list_shares_with_detail_filter_by_existed_name(self):
        # list shares by name, at least one share is expected
        params = {"name": self.share_name}
        shares = self.shares_client.list_shares_with_detail(
            params)['shares']
        self.assertEqual(self.share_name, shares[0]["name"])

    @decorators.idempotent_id('f446e8cb-5bef-45ac-8b87-f4136f44ca69')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported("2.36")
    def test_list_shares_with_detail_filter_by_existed_description(self):
        # list shares by description, at least one share is expected
        params = {"description": self.share_desc}
        shares = self.shares_v2_client.list_shares_with_detail(
            params)['shares']
        self.assertEqual(self.share_name, shares[0]["name"])

    @decorators.idempotent_id('1276b97b-cf46-4953-973f-f995985a1ce4')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported("2.36")
    def test_list_shares_with_detail_filter_by_inexact_name(self):
        # list shares by name, at least one share is expected
        params = {"name~": 'tempest-share'}
        shares = self.shares_v2_client.list_shares_with_detail(
            params)['shares']
        for share in shares:
            self.assertIn('tempest-share', share["name"])

    @decorators.idempotent_id('56416b95-949f-4b09-9a5e-377b674efd25')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_list_shares_with_detail_filter_by_fake_name(self):
        # list shares by fake name, no shares are expected
        params = {"name": data_utils.rand_name("fake-nonexistent-name")}
        shares = self.shares_client.list_shares_with_detail(params)['shares']
        self.assertEqual(0, len(shares))

    @decorators.idempotent_id('708e3e2e-8761-4d16-b18d-a834ee7ca69e')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_list_shares_with_detail_filter_by_active_status(self):
        # list shares by active status, at least one share is expected
        params = {"status": "available"}
        shares = self.shares_client.list_shares_with_detail(params)['shares']
        self.assertGreater(len(shares), 0)
        for share in shares:
            self.assertEqual(params["status"], share["status"])

    @decorators.idempotent_id('5ec2fcf8-18d4-4790-95de-45c8a05582c7')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_list_shares_with_detail_filter_by_fake_status(self):
        # list shares by fake status, no shares are expected
        params = {"status": 'fake'}
        shares = self.shares_client.list_shares_with_detail(params)['shares']
        self.assertEqual(0, len(shares))

    @decorators.idempotent_id('7609b7bb-613e-474d-a9b3-e41584842503')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_list_shares_with_detail_filter_by_all_tenants(self):
        # non-admin user can get shares only from his project
        params = {"all_tenants": 1}
        shares = self.shares_client.list_shares_with_detail(params)['shares']
        self.assertGreater(len(shares), 0)

        # get share with detailed info, we need its 'project_id'
        share = self.shares_client.get_share(self.shares[0]["id"])['share']
        project_id = share["project_id"]
        for share in shares:
            self.assertEqual(project_id, share["project_id"])

    @decorators.idempotent_id('0019afa2-fae2-417f-a7e0-2af665a966b0')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported("2.42")
    def test_list_shares_with_detail_with_count(self):
        # list shares by name, at least one share is expected
        params = {"with_count": 'true'}
        shares = self.shares_v2_client.list_shares_with_detail(params)
        self.assertGreater(shares["count"], 0)

    @decorators.idempotent_id('174829eb-fd3e-46ef-880b-f05c3d44d1fe')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @testtools.skipUnless(CONF.share.run_snapshot_tests,
                          "Snapshot tests are disabled.")
    @ddt.data(None, '2.16', LATEST_MICROVERSION)
    def test_get_snapshot(self, version):

        # get snapshot
        if version is None:
            snapshot = self.shares_client.get_snapshot(
                self.snap["id"])['snapshot']
        else:
            utils.check_skip_if_microversion_not_supported(version)
            snapshot = self.shares_v2_client.get_snapshot(
                self.snap["id"], version=version)['snapshot']

        # verify keys
        expected_keys = ["status", "links", "share_id", "name",
                         "share_proto", "created_at",
                         "description", "id", "share_size", "size"]
        if version and utils.is_microversion_ge(version, '2.17'):
            expected_keys.extend(["user_id", "project_id"])
        if version and utils.is_microversion_ge(version, '2.73'):
            expected_keys.extend(["metadata"])
        actual_keys = snapshot.keys()

        # strict key check
        self.assertEqual(set(expected_keys), set(actual_keys))

        # verify data
        msg = "Expected name: '%s', actual name: '%s'" % (self.snap_name,
                                                          snapshot["name"])
        self.assertEqual(self.snap_name, snapshot["name"], msg)

        msg = ("Expected description: '%s' actual description: '%s'" %
               (self.snap_desc, snapshot["description"]))
        self.assertEqual(self.snap_desc, snapshot["description"], msg)

        msg = ("Expected share_id: '%s', actual share_id: '%s'" %
               (self.shares[0]["id"], snapshot["share_id"]))
        self.assertEqual(self.shares[0]["id"], snapshot["share_id"], msg)

        # Verify that the user_id and project_id are same as the one for
        # the base share
        if version and utils.is_microversion_ge(version, '2.17'):
            msg = ("Expected %(key)s in snapshot: '%(expected)s', "
                   "actual %(key)s in snapshot: '%(actual)s'")
            self.assertEqual(self.shares[0]['user_id'],
                             snapshot['user_id'],
                             msg % {
                                 'expected': self.shares[0]['user_id'],
                                 'actual': snapshot['user_id'],
                                 'key': 'user_id'})
            self.assertEqual(self.shares[0]['project_id'],
                             snapshot['project_id'],
                             msg % {
                                 'expected': self.shares[0]['project_id'],
                                 'actual': snapshot['project_id'],
                                 'key': 'project_id'})

    @decorators.idempotent_id('3d5d85ff-6158-4af2-a765-cfa07a46adde')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @testtools.skipUnless(CONF.share.run_snapshot_tests,
                          "Snapshot tests are disabled.")
    def test_list_snapshots(self):

        # list share snapshots
        snaps = self.shares_client.list_snapshots()['snapshots']

        # verify keys
        keys = ["id", "name", "links"]
        [self.assertIn(key, sn.keys()) for sn in snaps for key in keys]

        # our share id in list and have no duplicates
        gen = [sid["id"] for sid in snaps if sid["id"] in self.snap["id"]]
        msg = "expected id lists %s times in share list" % (len(gen))
        self.assertEqual(1, len(gen), msg)

    @decorators.idempotent_id('4d717665-e4ca-47df-b4b9-f6d096159779')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @testtools.skipUnless(CONF.share.run_snapshot_tests,
                          "Snapshot tests are disabled.")
    @ddt.data(None, '2.16', '2.36', LATEST_MICROVERSION)
    def test_list_snapshots_with_detail(self, version):
        params = None
        if version and utils.is_microversion_ge(version, '2.36'):
            params = {'name~': 'tempest', 'description~': 'tempest'}
        # list share snapshots
        if version is None:
            snaps = self.shares_client.list_snapshots_with_detail(
                )['snapshots']
        else:
            utils.check_skip_if_microversion_not_supported(version)
            snaps = self.shares_v2_client.list_snapshots_with_detail(
                version=version, params=params)['snapshots']

        # verify keys
        expected_keys = ["status", "links", "share_id", "name",
                         "share_proto", "created_at", "description", "id",
                         "share_size", "size"]
        if version and utils.is_microversion_ge(version, '2.17'):
            expected_keys.extend(["user_id", "project_id"])
        if version and utils.is_microversion_ge(version, '2.73'):
            expected_keys.append("metadata")

        if version and utils.is_microversion_ge(version, '2.73'):
            expected_keys.extend(["metadata"])

        # strict key check
        [self.assertEqual(set(expected_keys), set(s.keys())) for s in snaps]

        # our share id in list and have no duplicates
        gen = [sid["id"] for sid in snaps if sid["id"] in self.snap["id"]]
        msg = "expected id lists %s times in share list" % (len(gen))
        self.assertEqual(1, len(gen), msg)

    @decorators.idempotent_id('80497a21-7533-47b5-93aa-29e0f7924cb9')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @testtools.skipUnless(CONF.share.run_snapshot_tests,
                          "Snapshot tests are disabled.")
    def test_list_snapshots_with_detail_use_limit(self):
        for limit, offset in [('1', '1'), ('0', '1')]:
            filters = {
                'limit': limit,
                'offset': offset,
                'share_id': self.shares[0]['id'],
            }

            # list snapshots
            snaps = self.shares_client.list_snapshots_with_detail(
                params=filters)['snapshots']

            # Our snapshot should not be listed
            self.assertEqual(0, len(snaps))

        # Only our one snapshot should be listed
        snaps = self.shares_client.list_snapshots_with_detail(
            params={'limit': '1', 'offset': '0',
                    'share_id': self.shares[0]['id']})['snapshots']

        self.assertEqual(1, len(snaps))
        self.assertEqual(self.snap['id'], snaps[0]['id'])

    @decorators.idempotent_id('0a94e996-c4db-4fef-b486-4004ea65c11a')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @testtools.skipUnless(CONF.share.run_snapshot_tests,
                          "Snapshot tests are disabled.")
    def test_list_snapshots_with_detail_filter_by_status_and_name(self):
        filters = {'status': 'available', 'name': self.snap_name}

        # list snapshots
        snaps = self.shares_client.list_snapshots_with_detail(
            params=filters)['snapshots']

        # verify response
        self.assertGreater(len(snaps), 0)
        for snap in snaps:
            self.assertEqual(filters['status'], snap['status'])
            self.assertEqual(filters['name'], snap['name'])

    @decorators.idempotent_id('f969aba1-d293-48e3-a638-a89785bb41ef')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @testtools.skipUnless(CONF.share.run_snapshot_tests,
                          "Snapshot tests are disabled.")
    @utils.skip_if_microversion_not_supported("2.35")
    def test_list_snapshots_with_detail_filter_by_description(self):
        filters = {'description': self.snap_desc}

        # list snapshots
        snaps = self.shares_client.list_snapshots_with_detail(
            params=filters)['snapshots']

        # verify response
        self.assertGreater(len(snaps), 0)
        for snap in snaps:
            self.assertEqual(filters['description'], snap['description'])

    @decorators.idempotent_id('59968026-12af-4029-a3d0-42c291b7db96')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @testtools.skipUnless(CONF.share.run_snapshot_tests,
                          "Snapshot tests are disabled.")
    def test_list_snapshots_with_detail_and_asc_sorting(self):
        filters = {'sort_key': 'share_id', 'sort_dir': 'asc'}

        # list snapshots
        snaps = self.shares_client.list_snapshots_with_detail(
            params=filters)['snapshots']

        # verify response
        self.assertGreater(len(snaps), 0)
        sorted_list = [snap['share_id'] for snap in snaps]
        self.assertEqual(sorted(sorted_list), sorted_list)

    @decorators.idempotent_id('17e6f579-e0d7-4724-a639-4974e82bb5ed')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    @testtools.skipUnless(
        CONF.share.run_extend_tests,
        "Share extend tests are disabled.")
    def test_extend_share(self):
        share = self.create_share(share_type_id=self.share_type_id,
                                  cleanup_in_class=False)
        new_size = int(share['size']) + 1

        # extend share and wait for active status
        self.shares_v2_client.extend_share(share['id'], new_size)
        waiters.wait_for_resource_status(
            self.shares_client, share['id'], 'available')

        # check state and new size
        share_get = self.shares_v2_client.get_share(share['id'])['share']
        msg = (
            "Share could not be extended. "
            "Expected %(expected)s, got %(actual)s." % {
                "expected": new_size,
                "actual": share_get['size'],
            }
        )
        self.assertEqual(new_size, share_get['size'], msg)

    @decorators.idempotent_id('8f64b930-9b4e-41d7-bbd6-82d9951931f3')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    @testtools.skipUnless(
        CONF.share.run_shrink_tests,
        "Share shrink tests are disabled.")
    def test_shrink_share(self):
        size = CONF.share.share_size + 1
        share = self.create_share(size=size,
                                  share_type_id=self.share_type_id,
                                  cleanup_in_class=False)
        new_size = int(share['size']) - 1

        # shrink share and wait for active status
        self.shares_v2_client.shrink_share(share['id'], new_size)
        waiters.wait_for_resource_status(
            self.shares_client, share['id'], 'available')

        # check state and new size
        share_get = self.shares_v2_client.get_share(share['id'])['share']
        msg = (
            "Share could not be shrunk. "
            "Expected %(expected)s, got %(actual)s." % {
                "expected": new_size,
                "actual": share_get['size'],
            }
        )
        self.assertEqual(new_size, share_get['size'], msg)

    @utils.skip_if_microversion_not_supported("2.69")
    @decorators.idempotent_id('7a19fb58-b645-44cc-a6d7-b3508ff8754d')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_soft_delete_and_restore_share(self):
        share = self.create_share(share_type_id=self.share_type_id)

        # list shares
        shares = self.shares_v2_client.list_shares()['shares']

        # check the share in share list
        share_ids = [sh['id'] for sh in shares]
        self.assertIn(share['id'], share_ids)

        # soft delete the share
        self.shares_v2_client.soft_delete_share(share['id'])
        waiters.wait_for_soft_delete(self.shares_v2_client, share['id'])

        # list shares again
        shares1 = self.shares_v2_client.list_shares()['shares']
        share_ids1 = [sh['id'] for sh in shares1]

        # list shares in recycle bin
        shares2 = self.shares_v2_client.list_shares_in_recycle_bin()['shares']
        share_ids2 = [sh['id'] for sh in shares2]

        # check share has been soft delete to recycle bin
        self.assertNotIn(share['id'], share_ids1)
        self.assertIn(share['id'], share_ids2)

        # restore share from recycle bin
        self.shares_v2_client.restore_share(share['id'])
        waiters.wait_for_restore(self.shares_v2_client, share['id'])

        # list shares again
        shares3 = self.shares_v2_client.list_shares()['shares']
        share_ids3 = [sh['id'] for sh in shares3]

        # list shares in recycle bin again
        shares4 = self.shares_v2_client.list_shares_in_recycle_bin()['shares']
        share_ids4 = [sh['id'] for sh in shares4]

        # check share has restored from recycle bin
        self.assertNotIn(share['id'], share_ids4)
        self.assertIn(share['id'], share_ids3)


class SharesRenameTest(base.BaseSharesMixedTest):

    @classmethod
    def resource_setup(cls):
        super(SharesRenameTest, cls).resource_setup()

        # create share_type
        extra_specs = {}
        if CONF.share.capability_snapshot_support:
            extra_specs.update({'snapshot_support': True})
        cls.share_type = cls.create_share_type(extra_specs=extra_specs)
        cls.share_type_id = cls.share_type['id']

        # create share
        cls.share_name = data_utils.rand_name("tempest-share-name")
        cls.share_desc = data_utils.rand_name("tempest-share-description")
        cls.share = cls.create_share(
            name=cls.share_name,
            description=cls.share_desc,
            share_type_id=cls.share_type_id)

        if CONF.share.run_snapshot_tests:
            # create snapshot
            cls.snap_name = data_utils.rand_name("tempest-snapshot-name")
            cls.snap_desc = data_utils.rand_name(
                "tempest-snapshot-description")
            cls.snap = cls.create_snapshot_wait_for_active(
                cls.share["id"], cls.snap_name, cls.snap_desc)

    @decorators.idempotent_id('7661d042-8222-483c-9249-9f53931e7347')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_update_share(self):

        # get share
        share = self.shares_client.get_share(self.share['id'])['share']
        self.assertEqual(self.share_name, share["name"])
        self.assertEqual(self.share_desc, share["description"])
        self.assertFalse(share["is_public"])

        # update share
        new_name = data_utils.rand_name("tempest-new-name")
        new_desc = data_utils.rand_name("tempest-new-description")
        updated = self.shares_client.update_share(
            share["id"], name=new_name, desc=new_desc)['share']
        self.assertEqual(new_name, updated["name"])
        self.assertEqual(new_desc, updated["description"])

        # get share
        share = self.shares_client.get_share(self.share['id'])['share']
        self.assertEqual(new_name, share["name"])
        self.assertEqual(new_desc, share["description"])
        self.assertFalse(share["is_public"])

    @decorators.idempotent_id('20f299f6-2441-4629-b44e-d791d57f413c')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @testtools.skipUnless(CONF.share.run_snapshot_tests,
                          "Snapshot tests are disabled.")
    def test_rename_snapshot(self):

        # get snapshot
        get = self.shares_client.get_snapshot(self.snap["id"])['snapshot']
        self.assertEqual(self.snap_name, get["name"])
        self.assertEqual(self.snap_desc, get["description"])

        # rename snapshot
        new_name = data_utils.rand_name("tempest-new-name-for-snapshot")
        new_desc = data_utils.rand_name("tempest-new-description-for-snapshot")
        renamed = self.shares_client.rename_snapshot(
            self.snap["id"], new_name, new_desc)['snapshot']
        self.assertEqual(new_name, renamed["name"])
        self.assertEqual(new_desc, renamed["description"])

        # get snapshot
        get = self.shares_client.get_snapshot(self.snap["id"])['snapshot']
        self.assertEqual(new_name, get["name"])
        self.assertEqual(new_desc, get["description"])



# Copyright 2019 NetApp Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ddt
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
import testtools
from testtools import testcase as tc

from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils

CONF = config.CONF


@ddt.ddt
class ManageShareServersTest(base.BaseSharesAdminTest):

    @classmethod
    def skip_checks(cls):
        super(ManageShareServersTest, cls).skip_checks()
        if not CONF.share.multitenancy_enabled:
            raise cls.skipException('Multitenancy tests are disabled.')
        if not CONF.share.run_manage_unmanage_tests:
            raise cls.skipException('Manage/unmanage tests are disabled.')

        utils.check_skip_if_microversion_not_supported('2.49')

    @classmethod
    def resource_setup(cls):
        super(ManageShareServersTest, cls).resource_setup()

        # create share type
        cls.st_name = data_utils.rand_name("manage-st-name")
        cls.extra_specs = {
            'storage_protocol': CONF.share.capability_storage_protocol,
            'driver_handles_share_servers': CONF.share.multitenancy_enabled,
        }
        cls.share_type = cls.create_share_type(
            name=cls.st_name,
            cleanup_in_class=True,
            extra_specs=cls.extra_specs)

    @decorators.idempotent_id('2bbe564d-d314-474b-8060-ada30581d0cd')
    @testtools.skipIf(CONF.share.share_network_id != "",
                      "This test is not suitable for pre-existing "
                      "share_network.")
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    @ddt.data(True, False)
    def test_manage_share_server(self, add_subnet_field):
        # Starting from v2.51 share network spans to multiple subnets.
        if add_subnet_field and not utils.is_microversion_supported('2.51'):
            msg = ("Manage share server with share network subnet is "
                   "supported starting from microversion '2.51'.")
            raise self.skipException(msg)
        check_multiple_subnet = utils.is_microversion_ge(
            CONF.share.max_api_microversion, '2.70')
        if check_multiple_subnet:
            network_subnet = 'share_network_subnet_ids'
        else:
            network_subnet = 'share_network_subnet_id'
        # create a new share network to make sure that a new share server
        # will be created
        original_share_network = self.shares_v2_client.get_share_network(
            self.shares_v2_client.share_network_id
        )['share_network']
        share_net_info = (
            utils.share_network_get_default_subnet(original_share_network)
            if utils.share_network_subnets_are_supported()
            else original_share_network)
        share_network = self.create_share_network(
            neutron_net_id=share_net_info['neutron_net_id'],
            neutron_subnet_id=share_net_info['neutron_subnet_id'],
            cleanup_in_class=True
        )
        az = params = None
        if add_subnet_field:
            # Get a compatible availability zone
            az = self.get_availability_zones_matching_share_type(
                self.share_type)[0]
            az_subnet = self.shares_v2_client.create_subnet(
                share_network['id'],
                neutron_net_id=share_network['neutron_net_id'],
                neutron_subnet_id=share_network['neutron_subnet_id'],
                availability_zone=az
            )['share_network_subnet']
            params = {network_subnet: az_subnet['id']}

        # create share
        share = self.create_share(
            share_type_id=self.share_type['id'],
            share_network_id=share_network['id'], availability_zone=az
        )
        share = self.shares_v2_client.get_share(share['id'])['share']
        el = self.shares_v2_client.list_share_export_locations(
            share['id'])['export_locations']
        share['export_locations'] = el
        share_server = self.shares_v2_client.show_share_server(
            share['share_server_id']
        )['share_server']

        keys = [
            "id",
            "host",
            "project_id",
            "status",
            "share_network_name",
            "created_at",
            "updated_at",
            "backend_details",
            "is_auto_deletable",
            "identifier",
        ]
        if add_subnet_field:
            keys.append(network_subnet)
        # all expected keys are present
        for key in keys:
            self.assertIn(key, share_server)

        # check that the share server is initially auto-deletable
        self.assertIs(True, share_server["is_auto_deletable"])
        self.assertIsNotNone(share_server["identifier"])
        if add_subnet_field and check_multiple_subnet:
            self.assertIn(az_subnet["id"], share_server[network_subnet])
        elif add_subnet_field and not check_multiple_subnet:
            self.assertEqual(az_subnet["id"], share_server[network_subnet])

        self._unmanage_share_and_wait(share)

        # Starting from microversion 2.49, any share server that has ever had
        # an unmanaged share will never be auto-deleted.
        share_server = self.shares_v2_client.show_share_server(
            share_server['id']
        )['share_server']
        self.assertIs(False, share_server['is_auto_deletable'])

        # unmanage share server and manage it again
        self._unmanage_share_server_and_wait(share_server)
        managed_share_server = self._manage_share_server(share_server,
                                                         fields=params)
        managed_share = self._manage_share(
            share,
            name="managed share that had ID %s" % share['id'],
            description="description for managed share",
            share_server_id=managed_share_server['id']
        )

        # check managed share server
        managed_share_server = self.shares_v2_client.show_share_server(
            managed_share_server['id']
        )['share_server']

        # all expected keys are present in the managed share server
        for key in keys:
            self.assertIn(key, managed_share_server)

        # check that managed share server is used by the managed share
        self.assertEqual(
            managed_share['share_server_id'],
            managed_share_server['id']
        )

        # check that the managed share server is still not auto-deletable
        self.assertIs(False, managed_share_server["is_auto_deletable"])

        # delete share
        self._delete_share_and_wait(managed_share)

        # delete share server
        self._delete_share_server_and_wait(managed_share_server['id'])

        if add_subnet_field:
            # delete the created subnet
            self.shares_v2_client.delete_subnet(share_network['id'],
                                                az_subnet['id'])



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from testtools import testcase as tc

from manila_tempest_tests.tests.api import base

CONF = config.CONF
LATEST_MICROVERSION = CONF.share.max_api_microversion


class PublicSharesTest(base.BaseSharesMixedTest):

    @classmethod
    def resource_setup(cls):
        super(PublicSharesTest, cls).resource_setup()
        # create share_type
        share_type = cls.create_share_type()
        cls.share_type_id = share_type['id']

    @decorators.idempotent_id('557a0474-9e30-47b4-a766-19e2afb13e66')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_list_shares_public_with_detail(self):
        # The default RBAC policy in manila only allows admin users with
        # system scope to create public shares since the Stein release
        public_share = self.create_share(
            name='public_share - must be visible to all projects in the cloud',
            description='public_share_desc',
            share_type_id=self.share_type_id,
            is_public=True,
            cleanup_in_class=False,
            client=self.admin_shares_v2_client,
            version=LATEST_MICROVERSION
        )
        private_share = self.create_share(
            name='private_share',
            description='private share in the primary user project',
            share_type_id=self.share_type_id,
            is_public=False,
            cleanup_in_class=False,
            version=LATEST_MICROVERSION
        )

        params = {'is_public': True}
        shares = self.alt_shares_v2_client.list_shares_with_detail(
            params)['shares']

        keys = [
            'status', 'description', 'links', 'availability_zone',
            'created_at', 'share_proto', 'name', 'snapshot_id', 'id',
            'size', 'project_id', 'is_public',
        ]
        [self.assertIn(key, sh.keys()) for sh in shares for key in keys]

        retrieved_public_share = [
            share for share in shares if share['id'] == public_share['id']
        ]
        msg = 'expected id lists %s times in share list' % (
            len(retrieved_public_share))
        self.assertEqual(1, len(retrieved_public_share), msg)
        self.assertTrue(retrieved_public_share[0]['is_public'])

        self.assertFalse(any([s['id'] == private_share['id'] for s in shares]))

    @decorators.idempotent_id('e073182e-459d-4e08-9300-5bc964ca806b')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_update_share_set_is_public(self):
        share_name = data_utils.rand_name('tempest-share-name')
        share = self.create_share(name=share_name,
                                  description='a share we will update',
                                  share_type_id=self.share_type_id,
                                  is_public=False,
                                  cleanup_in_class=False,
                                  version=LATEST_MICROVERSION)

        share = self.shares_v2_client.get_share(share['id'])['share']
        self.assertEqual(share_name, share['name'])
        self.assertEqual('a share we will update', share['description'])
        self.assertFalse(share['is_public'])

        # update share, manila's default RBAC only allows administrator
        # users with a system scope token to update a private share to public
        new_name = data_utils.rand_name('tempest-new-share-name')
        new_desc = 'share is now updated'
        updated = self.admin_shares_v2_client.update_share(
            share['id'], name=new_name, desc=new_desc, is_public=True)['share']
        self.assertEqual(new_name, updated['name'])
        self.assertEqual(new_desc, updated['description'])
        self.assertTrue(updated['is_public'])

        # this share must now be publicly accessible
        share = self.alt_shares_v2_client.get_share(share['id'])['share']
        self.assertEqual(new_name, share['name'])
        self.assertEqual(new_desc, share['description'])
        self.assertTrue(share['is_public'])



# Copyright 2015 Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from collections import OrderedDict
import random
import re

from netaddr import ip
from tempest import config
from tempest.lib.common.utils import data_utils
import testtools

from manila_tempest_tests import utils

CONF = config.CONF
SHARE_NETWORK_SUBNETS_MICROVERSION = '2.51'
SHARE_REPLICA_QUOTAS_MICROVERSION = "2.53"
EXPERIMENTAL = {'X-OpenStack-Manila-API-Experimental': 'True'}


def deduplicate(items):
    """De-duplicate a list of items while preserving the order.

    It is useful when passing a list of items to ddt.data, in order
    to remove duplicated elements which may be specified as constants.
    """
    return list(OrderedDict.fromkeys(items))


def get_microversion_as_tuple(microversion_str):
    """Transforms string-like microversion to two-value tuple of integers.

    Tuple of integers useful for microversion comparisons.
    """
    regex = r"^([1-9]\d*)\.([1-9]\d*|0)$"
    match = re.match(regex, microversion_str)
    if not match:
        raise ValueError(
            "Microversion does not fit template 'x.y' - %s" % microversion_str)
    return int(match.group(1)), int(match.group(2))


def is_microversion_gt(left, right):
    """Is microversion for left is greater than the right one."""
    return get_microversion_as_tuple(left) > get_microversion_as_tuple(right)


def is_microversion_ge(left, right):
    """Is microversion for left is greater than or equal to the right one."""
    return get_microversion_as_tuple(left) >= get_microversion_as_tuple(right)


def is_microversion_eq(left, right):
    """Is microversion for left is equal to the right one."""
    return get_microversion_as_tuple(left) == get_microversion_as_tuple(right)


def is_microversion_ne(left, right):
    """Is microversion for left is not equal to the right one."""
    return get_microversion_as_tuple(left) != get_microversion_as_tuple(right)


def is_microversion_le(left, right):
    """Is microversion for left is less than or equal to the right one."""
    return get_microversion_as_tuple(left) <= get_microversion_as_tuple(right)


def is_microversion_lt(left, right):
    """Is microversion for left is less than the right one."""
    return get_microversion_as_tuple(left) < get_microversion_as_tuple(right)


def is_microversion_supported(microversion):
    bottom = get_microversion_as_tuple(CONF.share.min_api_microversion)
    microversion = get_microversion_as_tuple(microversion)
    top = get_microversion_as_tuple(CONF.share.max_api_microversion)
    return bottom <= microversion <= top


def skip_if_microversion_not_supported(microversion):
    """Decorator for tests that are microversion-specific."""
    if not is_microversion_supported(microversion):
        reason = ("Skipped. Test requires microversion '%s'." % microversion)
        return testtools.skip(reason)
    return lambda f: f


def skip_if_is_microversion_ge(left, right):
    """Skip if version for left is greater than or equal to the right one."""

    if is_microversion_ge(left, right):
        reason = ("Skipped. Test requires microversion "
                  "< than '%s'." % right)
        return testtools.skip(reason)
    return lambda f: f


def check_skip_if_microversion_not_supported(microversion):
    """Callable method for tests that are microversion-specific."""
    if not is_microversion_supported(microversion):
        reason = ("Skipped. Test requires microversion '%s'." % microversion)
        raise testtools.TestCase.skipException(reason)


def rand_ip(network=False):
    """This uses the TEST-NET-3 range of reserved IP addresses.

    Using this range, which are reserved solely for use in
    documentation and example source code, should avoid any potential
    conflicts in real-world testing.
    """
    test_net_3 = '203.0.113.'
    address = test_net_3 + str(random.randint(0, 255))
    if network:
        mask_length = str(random.randint(24, 32))
        address = '/'.join((address, mask_length))
        ip_network = ip.IPNetwork(address)
        return '/'.join((str(ip_network.network), mask_length))
    return address


def rand_ipv6_ip(network=False):
    """This uses the IPv6 documentation range of 2001:DB8::/32"""
    ran_add = ["%x" % random.randrange(0, 16 ** 4) for i in range(6)]
    address = "2001:0DB8:" + ":".join(ran_add)
    if network:
        mask_length = str(random.randint(32, 128))
        address = '/'.join((address, mask_length))
        ip_network = ip.IPNetwork(address)
        return '/'.join((str(ip_network.network), mask_length))
    return address


def generate_share_network_data():
    data = {
        "name": data_utils.rand_name("sn-name"),
        "description": data_utils.rand_name("sn-desc"),
        "neutron_net_id": data_utils.rand_name("net-id"),
        "neutron_subnet_id": data_utils.rand_name("subnet-id"),
    }
    return data


def generate_subnet_data():
    data = {
        "neutron_net_id": data_utils.rand_name("net-id"),
        "neutron_subnet_id": data_utils.rand_name("subnet-id"),
    }
    return data


def generate_security_service_data(set_ou=False):
    data = {
        "name": data_utils.rand_name("ss-name"),
        "description": data_utils.rand_name("ss-desc"),
        "dns_ip": utils.rand_ip(),
        "server": utils.rand_ip(),
        "domain": data_utils.rand_name("ss-domain"),
        "user": data_utils.rand_name("ss-user"),
        "password": data_utils.rand_name("ss-password"),
    }
    if set_ou:
        data["ou"] = data_utils.rand_name("ss-ou")

    return data


def choose_matching_backend(share, pools, share_type):
    extra_specs = {}
    # fix extra specs with string values instead of boolean
    for k, v in share_type['extra_specs'].items():
        extra_specs[k] = (True if str(v).lower() == 'true'
                          else False if str(v).lower() == 'false'
                          else v)
    selected_pool = next(
        (x for x in pools if (x['name'] != share['host'] and all(
            y in x['capabilities'].items() for y in extra_specs.items()))),
        None)

    return selected_pool


def get_configured_extra_specs(variation=None):
    """Retrieve essential extra specs according to configuration in tempest.

    :param variation: can assume possible values: None to be as configured in
        tempest; 'opposite_driver_modes' for as configured in tempest but
        inverse driver mode; 'invalid' for inverse as configured in tempest,
        ideal for negative tests.
    :return: dict containing essential extra specs.
    """

    extra_specs = {'storage_protocol': CONF.share.capability_storage_protocol}

    if variation == 'invalid':
        extra_specs['driver_handles_share_servers'] = (
            not CONF.share.multitenancy_enabled)
        extra_specs['snapshot_support'] = (
            not CONF.share.capability_snapshot_support)

    elif variation == 'opposite_driver_modes':
        extra_specs['driver_handles_share_servers'] = (
            not CONF.share.multitenancy_enabled)
        extra_specs['snapshot_support'] = (
            CONF.share.capability_snapshot_support)

    else:
        extra_specs['driver_handles_share_servers'] = (
            CONF.share.multitenancy_enabled)
        extra_specs['snapshot_support'] = (
            CONF.share.capability_snapshot_support)
        extra_specs['create_share_from_snapshot_support'] = (
            CONF.share.capability_create_share_from_snapshot_support)

    return extra_specs


def get_access_rule_data_from_config(protocol):
    """Get the first available access type/to combination from config.

    This method opportunistically picks the first configured protocol
    to create the share. Do not use this method in tests where you need
    to test depth and breadth in the access types and access recipients.
    """

    if protocol in CONF.share.enable_ip_rules_for_protocols:
        access_type = "ip"
        access_to = rand_ip()
    elif protocol in CONF.share.enable_user_rules_for_protocols:
        access_type = "user"
        access_to = CONF.share.username_for_user_rules
    elif protocol in CONF.share.enable_cert_rules_for_protocols:
        access_type = "cert"
        access_to = "client3.com"
    elif protocol in CONF.share.enable_cephx_rules_for_protocols:
        access_type = "cephx"
        access_to = data_utils.rand_name("cephx-id")
    else:
        message = "Unrecognized protocol and access rules configuration."
        raise testtools.TestCase.skipException(message)

    return access_type, access_to


def replication_with_multitenancy_support():
    return (share_network_subnets_are_supported() and
            CONF.share.multitenancy_enabled)


def skip_if_manage_not_supported_for_version(
        version=CONF.share.max_api_microversion):
    if (is_microversion_lt(version, "2.49")
            and CONF.share.multitenancy_enabled):
        raise testtools.TestCase.skipException(
            "Share manage tests with multitenancy are disabled for "
            "microversion < 2.49")


def share_network_subnets_are_supported():
    return is_microversion_supported(SHARE_NETWORK_SUBNETS_MICROVERSION)


def share_replica_quotas_are_supported():
    return is_microversion_supported(SHARE_REPLICA_QUOTAS_MICROVERSION)


def share_network_get_default_subnet(share_network):
    return next((
        subnet for subnet in share_network.get('share_network_subnets', [])
        if subnet['availability_zone'] is None), None)


def get_extra_headers(request_version, graduation_version):
    headers = None
    extra_headers = False
    if is_microversion_lt(request_version, graduation_version):
        headers = EXPERIMENTAL
        extra_headers = True
    return headers, extra_headers



# -*- coding: utf-8 -*-
# Copyright 2016 Andrew Kerr
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ddt
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
import testtools
from testtools import testcase as tc

from manila_tempest_tests.common import constants
from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils

CONF = config.CONF
LATEST_MICROVERSION = CONF.share.max_api_microversion


@ddt.ddt
class ShareGroupActionsTest(base.BaseSharesMixedTest):
    """Covers share group functionality."""

    @classmethod
    def skip_checks(cls):
        super(ShareGroupActionsTest, cls).skip_checks()
        if not CONF.share.run_share_group_tests:
            raise cls.skipException('Share Group tests disabled.')

        utils.check_skip_if_microversion_not_supported(
            constants.MIN_SHARE_GROUP_MICROVERSION)

    @classmethod
    def resource_setup(cls):
        super(ShareGroupActionsTest, cls).resource_setup()

        # Create a share type
        extra_specs = {}
        if CONF.share.capability_snapshot_support:
            extra_specs.update({'snapshot_support': True})
        if CONF.share.capability_create_share_from_snapshot_support:
            extra_specs.update({'create_share_from_snapshot_support': True})
        cls.share_type = cls.create_share_type(extra_specs=extra_specs)
        cls.share_type_id = cls.share_type['id']

        cls.share_group_type = cls._create_share_group_type()
        cls.share_group_type_id = cls.share_group_type['id']

        # Create first share group
        cls.share_group_name = data_utils.rand_name("tempest-sg-name")
        cls.share_group_desc = data_utils.rand_name("tempest-sg-description")
        cls.share_group = cls.create_share_group(
            name=cls.share_group_name,
            description=cls.share_group_desc,
            share_group_type_id=cls.share_group_type_id,
            share_type_ids=[cls.share_type_id],
        )

        # Create second share group for purposes of sorting and snapshot
        # filtering
        cls.share_group2 = cls.create_share_group(
            name=cls.share_group_name,
            description=cls.share_group_desc,
            share_group_type_id=cls.share_group_type_id,
            share_type_ids=[cls.share_type_id],
        )

        # Create 2 shares - inside first and second share groups
        cls.share_name = data_utils.rand_name("tempest-share-name")
        cls.share_desc = data_utils.rand_name("tempest-share-description")
        cls.share_size = CONF.share.share_size
        cls.share_size2 = CONF.share.share_size + 2
        cls.shares = cls.create_shares([
            {'kwargs': {
                'name': cls.share_name,
                'description': cls.share_desc,
                'size': size,
                'share_type_id': cls.share_type_id,
                'share_group_id': sg_id,
            }} for size, sg_id in ((cls.share_size, cls.share_group['id']),
                                   (cls.share_size2, cls.share_group['id']),
                                   (cls.share_size, cls.share_group2['id']))
        ])

        # Create share group snapshots
        if CONF.share.capability_snapshot_support:
            cls.sg_snap_name = data_utils.rand_name("tempest-sg-snap-name")
            cls.sg_snap_desc = data_utils.rand_name("tempest-sg-snap-desc")

            cls.sg_snapshot = cls.create_share_group_snapshot_wait_for_active(
                cls.share_group["id"],
                name=cls.sg_snap_name,
                description=cls.sg_snap_desc,
            )

            cls.sg_snapshot2 = cls.create_share_group_snapshot_wait_for_active(
                cls.share_group2['id'],
                name=cls.sg_snap_name,
                description=cls.sg_snap_desc,
            )

    @decorators.idempotent_id('1e359389-09a7-4235-84c9-7b5c83632fff')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @ddt.data(
        *utils.deduplicate([constants.MIN_SHARE_GROUP_MICROVERSION,
                            constants.SHARE_GROUPS_GRADUATION_VERSION,
                            LATEST_MICROVERSION]))
    def test_get_share_group(self, version):
        utils.check_skip_if_microversion_not_supported(version)

        # Get share group
        share_group = self.shares_v2_client.get_share_group(
            self.share_group['id'], version=version)['share_group']

        # Verify keys
        actual_keys = set(share_group.keys())
        self.assertTrue(
            constants.SHARE_GROUP_DETAIL_REQUIRED_KEYS.issubset(actual_keys),
            'Not all required keys returned for share group %s. '
            'Expected at least: %s, found %s' % (
                share_group['id'],
                constants.SHARE_GROUP_DETAIL_REQUIRED_KEYS,
                actual_keys))

        # Verify values
        self.assertEqual(self.share_group_name, share_group["name"])
        self.assertEqual(self.share_group_desc, share_group["description"])

    @decorators.idempotent_id('45b77673-b1bb-43a1-b4b7-41351930adbd')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_get_share_min_supported_sg_microversion(self):

        # Get share
        share = self.shares_v2_client.get_share(
            self.shares[0]['id'],
            version=constants.MIN_SHARE_GROUP_MICROVERSION)['share']

        # Verify keys
        expected_keys = {
            "status", "description", "links", "availability_zone",
            "created_at", "share_proto", "name", "snapshot_id",
            "id", "size", "share_group_id",
        }
        actual_keys = set(share.keys())
        self.assertTrue(
            expected_keys.issubset(actual_keys),
            'Not all required keys returned for share %s.  '
            'Expected at least: %s, found %s' % (
                share['id'], expected_keys, actual_keys))

        # Verify values
        self.assertEqual(self.share_name, share["name"])
        self.assertEqual(self.share_desc, share["description"])
        self.assertEqual(self.share_size, int(share["size"]))
        self.assertEqual(self.share_group["id"], share["share_group_id"])

    @decorators.idempotent_id('04fcd695-c5f8-4de7-ab09-131424e6bdfb')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @ddt.data(
        *utils.deduplicate([constants.MIN_SHARE_GROUP_MICROVERSION,
                            constants.SHARE_GROUPS_GRADUATION_VERSION,
                            LATEST_MICROVERSION]))
    def test_list_share_groups(self, version):
        utils.check_skip_if_microversion_not_supported(version)

        # List share groups
        share_groups = self.shares_v2_client.list_share_groups(
            version=version)['share_groups']

        # Verify keys
        self.assertGreater(len(share_groups), 0)
        for sg in share_groups:
            keys = set(sg.keys())
            self.assertEqual(
                constants.SHARE_GROUP_SIMPLE_KEYS,
                keys,
                'Incorrect keys returned for share group %s. '
                'Expected: %s, found %s' % (
                    sg['id'],
                    constants.SHARE_GROUP_SIMPLE_KEYS,
                    ','.join(keys)))

        # Share group ids are in list exactly once
        for sg_id in (self.share_group["id"], self.share_group2["id"]):
            gen = [sg["id"] for sg in share_groups if sg["id"] == sg_id]
            msg = ("Expected id %s exactly once in share group list" % sg_id)
            self.assertEqual(1, len(gen), msg)

    @decorators.idempotent_id('16986c21-ecbc-429e-ab3d-8d1596a3eac4')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @ddt.data(
        *utils.deduplicate([constants.MIN_SHARE_GROUP_MICROVERSION, '2.36',
                            constants.SHARE_GROUPS_GRADUATION_VERSION,
                            LATEST_MICROVERSION]))
    def test_list_share_groups_with_detail_min(self, version):
        utils.check_skip_if_microversion_not_supported(version)
        params = None
        if utils.is_microversion_ge(version, '2.36'):
            params = {'name~': 'tempest', 'description~': 'tempest'}
        # List share groups
        share_groups = self.shares_v2_client.list_share_groups(
            detailed=True, params=params, version=version)['share_groups']

        # Verify keys
        for sg in share_groups:
            keys = set(sg.keys())
            self.assertTrue(
                constants.SHARE_GROUP_DETAIL_REQUIRED_KEYS.issubset(
                    keys),
                'Not all required keys returned for share group %s.  '
                'Expected at least: %s, found %s' % (
                    sg['id'],
                    constants.SHARE_GROUP_DETAIL_REQUIRED_KEYS,
                    ','.join(keys),
                )
            )

        # Share group ids are in list exactly once
        for group_id in (self.share_group["id"], self.share_group2["id"]):
            gen = [share_group["id"] for share_group in share_groups
                   if share_group["id"] == group_id]
            msg = ("Expected id %s exactly once in share group list" %
                   group_id)
            self.assertEqual(1, len(gen), msg)

    @decorators.idempotent_id('e72be2f9-56db-467f-89d7-0dddbf7e37e9')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_filter_shares_by_share_group_id_min(self):
        shares = self.shares_v2_client.list_shares(
            detailed=True,
            params={'share_group_id': self.share_group['id']},
            version=constants.MIN_SHARE_GROUP_MICROVERSION,
        )['shares']

        share_ids = [share['id'] for share in shares]

        self.assertEqual(
            2, len(shares),
            'Incorrect number of shares returned. '
            'Expected 2, got %s' % len(shares))
        self.assertIn(
            self.shares[0]['id'], share_ids,
            'Share %s expected in returned list, but got %s' % (
                self.shares[0]['id'], share_ids))
        self.assertIn(
            self.shares[1]['id'], share_ids,
            'Share %s expected in returned list, but got %s' % (
                self.shares[0]['id'], share_ids))

    @decorators.idempotent_id('5d2ca4f5-04da-4528-af47-ec980b95e884')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @ddt.data(
        *utils.deduplicate([constants.MIN_SHARE_GROUP_MICROVERSION,
                            constants.SHARE_GROUPS_GRADUATION_VERSION,
                            LATEST_MICROVERSION]))
    @testtools.skipUnless(CONF.share.run_snapshot_tests,
                          "Snapshot tests are disabled.")
    def test_get_share_group_snapshot(self, version):
        utils.check_skip_if_microversion_not_supported(version)

        # Get share group snapshot
        sg_snapshot = self.shares_v2_client.get_share_group_snapshot(
            self.sg_snapshot['id'],
            version=version,
        )['share_group_snapshot']

        # Verify keys
        actual_keys = set(sg_snapshot.keys())
        self.assertTrue(
            constants.SHARE_GROUP_SNAPSHOT_DETAIL_REQUIRED_KEYS.issubset(
                actual_keys),
            'Not all required keys returned for share group %s.  '
            'Expected at least: %s, found %s' % (
                sg_snapshot['id'],
                constants.SHARE_GROUP_DETAIL_REQUIRED_KEYS,
                actual_keys,
            )
        )

        # Verify values
        self.assertEqual(self.sg_snap_name, sg_snapshot["name"])
        self.assertEqual(self.sg_snap_desc, sg_snapshot["description"])

    @decorators.idempotent_id('67e8c099-f1c1-4972-9c51-bb7bfe1d7994')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @testtools.skipUnless(CONF.share.run_snapshot_tests,
                          "Snapshot tests are disabled.")
    def test_get_share_group_snapshot_members_min(self):
        sg_snapshot = self.shares_v2_client.get_share_group_snapshot(
            self.sg_snapshot['id'],
            version=constants.MIN_SHARE_GROUP_MICROVERSION,
        )['share_group_snapshot']
        sg_snapshot_members = sg_snapshot['members']
        member_share_ids = [m['share_id'] for m in sg_snapshot_members]
        self.assertEqual(
            2, len(sg_snapshot_members),
            'Unexpected number of share group snapshot members. '
            'Expected 2, got %s.' % len(sg_snapshot_members))
        # Verify each share is represented in the share group snapshot
        # appropriately
        for share_id in (self.shares[0]['id'], self.shares[1]['id']):
            self.assertIn(
                share_id, member_share_ids,
                'Share missing %s missing from share group '
                'snapshot. Found %s.' % (share_id, member_share_ids))
        for share in (self.shares[0], self.shares[1]):
            for member in sg_snapshot_members:
                if share['id'] == member['share_id']:
                    self.assertEqual(share['size'], member['size'])

    @decorators.idempotent_id('650c5fa7-11f2-48bd-b012-fc2e32b6f446')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @ddt.data(
        *utils.deduplicate([constants.MIN_SHARE_GROUP_MICROVERSION,
                            constants.SHARE_GROUPS_GRADUATION_VERSION,
                            LATEST_MICROVERSION]))
    @testtools.skipUnless(CONF.share.run_snapshot_tests,
                          "Snapshot tests are disabled.")
    @testtools.skipUnless(
        CONF.share.capability_create_share_from_snapshot_support,
        "Tests creating shares from snapshots are disabled.")
    def test_create_share_group_from_populated_share_group_snapshot(self,
                                                                    version):
        utils.check_skip_if_microversion_not_supported(version)

        sg_snapshot = self.shares_v2_client.get_share_group_snapshot(
            self.sg_snapshot['id'],
            version=version,
        )['share_group_snapshot']
        snapshot_members = sg_snapshot['members']

        new_share_group = self.create_share_group(
            cleanup_in_class=False,
            source_share_group_snapshot_id=self.sg_snapshot['id'],
            version=version,
            share_group_type_id=self.share_group_type_id,
        )

        new_share_group = self.shares_v2_client.get_share_group(
            new_share_group['id'],
            version=version,
        )['share_group']

        # Verify that share_network information matches source share group
        self.assertEqual(
            self.share_group['share_network_id'],
            new_share_group['share_network_id'])

        new_shares = self.shares_v2_client.list_shares(
            params={'share_group_id': new_share_group['id']},
            detailed=True,
            version=version,
        )['shares']

        # Verify each new share is available
        for share in new_shares:
            self.assertEqual(
                'available', share['status'],
                'Share %s is not in available status.' % share['id'])

        # Verify each sgsnapshot member is represented in the new sg
        # appropriately
        share_source_member_ids = [
            share['source_share_group_snapshot_member_id']
            for share in new_shares]
        for member in snapshot_members:
            self.assertIn(
                member['id'], share_source_member_ids,
                'Share group snapshot member %s not represented by '
                'share group %s.' % (member['id'], new_share_group['id']))
            for share in new_shares:
                if (share['source_share_group_snapshot_member_id'] == (
                        member['id'])):
                    self.assertEqual(member['size'], share['size'])
                    self.assertEqual(
                        self.share_group['share_network_id'],
                        share['share_network_id'])


@ddt.ddt
class ShareGroupRenameTest(base.BaseSharesMixedTest):

    @classmethod
    def skip_checks(cls):
        super(ShareGroupRenameTest, cls).skip_checks()
        if not CONF.share.run_share_group_tests:
            raise cls.skipException('Share Group tests disabled.')

        utils.check_skip_if_microversion_not_supported(
            constants.MIN_SHARE_GROUP_MICROVERSION)

    @classmethod
    def resource_setup(cls):
        super(ShareGroupRenameTest, cls).resource_setup()

        # Create a share type
        cls.share_type = cls.create_share_type()
        cls.share_type_id = cls.share_type['id']

        # Create a share group type
        cls.share_group_type = cls._create_share_group_type()
        cls.share_group_type_id = cls.share_group_type['id']

        # Create share group
        cls.share_group_name = data_utils.rand_name("tempest-sg-name")
        cls.share_group_desc = data_utils.rand_name("tempest-sg-description")
        cls.share_group = cls.create_share_group(
            name=cls.share_group_name,
            description=cls.share_group_desc,
            share_group_type_id=cls.share_group_type_id,
            share_type_ids=[cls.share_type_id]
        )

    def _rollback_share_group_update(self, version):
        self.shares_v2_client.update_share_group(
            self.share_group["id"],
            name=self.share_group_name,
            description=self.share_group_desc,
            version=version,
        )

    @decorators.idempotent_id('7f0a07ce-afdd-4c51-a29c-d8fe6cb5f6a5')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @ddt.data(
        *utils.deduplicate([constants.MIN_SHARE_GROUP_MICROVERSION,
                            constants.SHARE_GROUPS_GRADUATION_VERSION,
                            LATEST_MICROVERSION]))
    def test_update_share_group(self, version):
        utils.check_skip_if_microversion_not_supported(version)

        # Get share_group
        share_group = self.shares_v2_client.get_share_group(
            self.share_group['id'],
            version=version
        )['share_group']
        self.assertEqual(self.share_group_name, share_group["name"])
        self.assertEqual(self.share_group_desc, share_group["description"])

        # Update share_group
        new_name = data_utils.rand_name("tempest-new-name")
        new_desc = data_utils.rand_name("tempest-new-description")
        updated = self.shares_v2_client.update_share_group(
            share_group["id"],
            name=new_name,
            description=new_desc,
            version=version,
        )['share_group']
        self.assertEqual(new_name, updated["name"])
        self.assertEqual(new_desc, updated["description"])

        # Get share_group
        share_group = self.shares_v2_client.get_share_group(
            self.share_group['id'],
            version=version,
        )['share_group']
        self.assertEqual(new_name, share_group["name"])
        self.assertEqual(new_desc, share_group["description"])

        # Rollback the update since this is a ddt and the class resources are
        # going to be reused
        self._rollback_share_group_update(version)

    @decorators.idempotent_id('611b1555-df09-499b-8aef-e8261e3f6863')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @ddt.data(
        *utils.deduplicate([constants.MIN_SHARE_GROUP_MICROVERSION,
                            constants.SHARE_GROUPS_GRADUATION_VERSION,
                            LATEST_MICROVERSION]))
    def test_create_update_read_share_group_with_unicode(self, version):
        utils.check_skip_if_microversion_not_supported(version)

        value1 = 'ಠ_ಠ'
        value2 = 'ಠ_ರೃ'

        # Create share_group
        share_group = self.create_share_group(
            cleanup_in_class=False,
            name=value1,
            description=value1,
            version=version,
            share_group_type_id=self.share_group_type_id,
            share_type_ids=[self.share_type_id]
        )
        self.assertEqual(value1, share_group["name"])
        self.assertEqual(value1, share_group["description"])

        # Update share group
        updated = self.shares_v2_client.update_share_group(
            share_group["id"],
            name=value2,
            description=value2,
            version=version,
        )['share_group']
        self.assertEqual(value2, updated["name"])
        self.assertEqual(value2, updated["description"])

        # Get share group
        share_group = self.shares_v2_client.get_share_group(
            share_group['id'], version=version)['share_group']
        self.assertEqual(value2, share_group["name"])
        self.assertEqual(value2, share_group["description"])

        # Rollback the update since this is a ddt and the class resources are
        # going to be reused
        self._rollback_share_group_update(version)



# Copyright 2014 Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest import config
from tempest.lib import decorators
from testtools import testcase as tc

from manila_tempest_tests.tests.api import base

CONF = config.CONF


class SharesMetadataTest(base.BaseSharesMixedTest):

    @classmethod
    def resource_setup(cls):
        super(SharesMetadataTest, cls).resource_setup()
        # create share type
        cls.share_type = cls.create_share_type()
        cls.share_type_id = cls.share_type['id']

        # create share
        cls.share = cls.create_share(share_type_id=cls.share_type_id)

    def _verify_share_metadata(self, share, md):

        # verify metadata items
        for key in md:
            get_value = self.shares_v2_client.get_metadata_item(
                share["id"], key)['meta']
            self.assertEqual(md[key], get_value[key])

    @decorators.idempotent_id('9070249f-6e94-4a38-a036-08debee547c3')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_set_metadata_in_share_creation(self):

        md = {"key1": "value1", "key2": "value2", }

        # create share with metadata
        share = self.create_share(share_type_id=self.share_type_id,
                                  metadata=md,
                                  cleanup_in_class=False)

        # verify metadata
        self._verify_share_metadata(share, md)

    @decorators.idempotent_id('2725ab8e-cc04-4032-9393-74726ba43eb7')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_set_get_delete_metadata(self):

        md = {"key3": "value3", "key4": "value4", "key.5.1": "value.5"}

        # create share
        share = self.create_share(share_type_id=self.share_type_id,
                                  cleanup_in_class=False)

        # set metadata
        self.shares_v2_client.set_metadata(share["id"], md)

        # verify metadata
        self._verify_share_metadata(share, md)

        # delete metadata
        for key in md.keys():
            self.shares_v2_client.delete_metadata(share["id"], key)

        # verify deletion of metadata
        get_metadata = self.shares_v2_client.get_metadata(share["id"])[
            'metadata']
        for key in md.keys():
            self.assertNotIn(key, list(get_metadata.keys()))

    @decorators.idempotent_id('4e5f8159-62b6-4d5c-f729-d8b1f029d7de')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_set_metadata_not_delete_pre_metadata(self):
        md1 = {"key9": "value9", "key10": "value10", }
        md2 = {"key11": "value11", "key12": "value12", }

        # create share
        share = self.create_share(share_type_id=self.share_type_id,
                                  cleanup_in_class=False)

        # set metadata
        self.shares_v2_client.set_metadata(share["id"], md1)

        # verify metadata
        self._verify_share_metadata(share, md1)

        # set metadata again
        self.shares_v2_client.set_metadata(share["id"], md2)

        # verify metadata
        md1.update(md2)
        md = md1

        # verify metadata
        self._verify_share_metadata(share, md)

        # delete metadata
        for key in md.keys():
            self.shares_v2_client.delete_metadata(share["id"], key)

        # verify deletion of metadata
        get_metadata = self.shares_v2_client.get_metadata(
            share["id"])['metadata']
        for key in md.keys():
            self.assertNotIn(key, list(get_metadata.keys()))

    @decorators.idempotent_id('2ec70ba5-050b-3b17-c862-c149e53543c0')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_set_metadata_key_already_exist(self):
        md1 = {"key9": "value9", "key10": "value10", }
        md2 = {"key9": "value13", "key11": "value11", }

        # create share
        share = self.create_share(share_type_id=self.share_type_id,
                                  cleanup_in_class=False)

        # set metadata
        self.shares_v2_client.set_metadata(share["id"], md1)

        # verify metadata
        self._verify_share_metadata(share, md1)

        # set metadata again
        self.shares_v2_client.set_metadata(share["id"], md2)

        # verify metadata
        md = {"key9": "value13", "key10": "value10",
              "key11": "value11"}
        self._verify_share_metadata(share, md)

        # delete metadata
        for key in md.keys():
            self.shares_v2_client.delete_metadata(share["id"], key)

        # verify deletion of metadata
        get_metadata = self.shares_v2_client.get_metadata(
            share["id"])['metadata']
        for key in md.keys():
            self.assertNotIn(key, list(get_metadata.keys()))

    @decorators.idempotent_id('c94851f4-2559-4712-9297-9912db1da7ff')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_set_and_update_metadata_by_key(self):

        md1 = {"key5": "value5", "key6": "value6", }
        md2 = {"key7": "value7", "key8": "value8", }

        # create share
        share = self.create_share(share_type_id=self.share_type_id,
                                  cleanup_in_class=False)

        # set metadata
        self.shares_v2_client.set_metadata(share["id"], md1)

        # update metadata
        self.shares_v2_client.update_all_metadata(share["id"], md2)

        # verify metadata
        self._verify_share_metadata(share, md2)

    @decorators.idempotent_id('698ba406-493f-4c69-a093-273676fed438')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_set_metadata_min_size_key(self):
        data = {"k": "value"}

        self.shares_v2_client.set_metadata(self.share["id"], data)

        body_get = self.shares_v2_client.get_metadata(
            self.share["id"])['metadata']
        self.assertEqual(data['k'], body_get.get('k'))

    @decorators.idempotent_id('34c5bd96-ced7-42ef-a114-570cc63cf81d')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_set_metadata_max_size_key(self):
        max_key = "k" * 255
        data = {max_key: "value"}

        self.shares_v2_client.set_metadata(self.share["id"], data)

        body_get = self.shares_v2_client.get_metadata(
            self.share["id"])['metadata']
        self.assertIn(max_key, body_get)
        self.assertEqual(data[max_key], body_get.get(max_key))

    @decorators.idempotent_id('c776ab40-f75e-4d9f-abcf-a8f628a25991')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_set_metadata_min_size_value(self):
        data = {"key": "v"}

        self.shares_v2_client.set_metadata(self.share["id"], data)

        body_get = self.shares_v2_client.get_metadata(
            self.share["id"])['metadata']
        self.assertEqual(data['key'], body_get['key'])

    @decorators.idempotent_id('759ec4ab-2537-44ad-852b-1af85c6ca933')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_set_metadata_max_size_value(self):
        max_value = "v" * 1023
        data = {"key": max_value}

        self.shares_v2_client.set_metadata(self.share["id"], data)

        body_get = self.shares_v2_client.get_metadata(
            self.share["id"])['metadata']
        self.assertEqual(data['key'], body_get['key'])

    @decorators.idempotent_id('c5ca19ba-3595-414a-8ff9-fbc88cd801ba')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_upd_metadata_min_size_key(self):
        data = {"k": "value"}

        self.shares_v2_client.update_all_metadata(self.share["id"], data)

        body_get = self.shares_v2_client.get_metadata(
            self.share["id"])['metadata']
        self.assertEqual(data["k"], body_get["k"])

    @decorators.idempotent_id('5eff5619-b7cd-42f1-85e0-47d3d47098dd')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_upd_metadata_max_size_key(self):
        max_key = "k" * 255
        data = {max_key: "value"}

        self.shares_v2_client.update_all_metadata(self.share["id"], data)

        body_get = self.shares_v2_client.get_metadata(
            self.share["id"])['metadata']
        body_get_keys = list(body_get.keys())
        self.assertIn(max_key, body_get_keys)
        self.assertEqual(data[max_key], body_get[max_key])

    @decorators.idempotent_id('44a572f1-6b5c-49d0-8f2e-1583ec3428d8')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_upd_metadata_min_size_value(self):
        data = {"key": "v"}

        self.shares_v2_client.update_all_metadata(self.share["id"], data)

        body_get = self.shares_v2_client.get_metadata(
            self.share["id"])['metadata']
        self.assertEqual(data["key"], body_get["key"])

    @decorators.idempotent_id('694d95e1-ba8c-49fc-a888-6f9f0d51d77d')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_upd_metadata_max_size_value(self):
        max_value = "v" * 1023
        data = {"key": max_value}

        self.shares_v2_client.update_all_metadata(self.share["id"], data)

        body_get = self.shares_v2_client.get_metadata(
            self.share["id"])['metadata']
        self.assertEqual(max_value, body_get["key"])


class SharesMetadataCEPHFSTest(base.BaseSharesMixedTest):

    protocol = "cephfs"

    @classmethod
    def resource_setup(cls):
        super(SharesMetadataCEPHFSTest, cls).resource_setup()
        # create share type
        cls.share_type = cls.create_share_type()
        cls.share_type_id = cls.share_type['id']
        # create share
        cls.share = cls.create_share(share_type_id=cls.share_type_id)

    @classmethod
    def skip_checks(cls):
        super(SharesMetadataCEPHFSTest, cls).skip_checks()
        if not (cls.protocol in CONF.share.enable_protocols):
            msg = (
                "CEPHFS filesystem metadata tests are disabled "
                "for the %s protocol." % cls.protocol)
            raise cls.skipException(msg)

    @decorators.idempotent_id('58edc9c8-8b85-49aa-80aa-209fc8f40a13')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_cephfs_share_contains_mount_option(self):
        body_get = self.shares_v2_client.get_metadata(
            self.share["id"])['metadata']

        self.assertIn("__mount_options", body_get)
        self.assertIn("fs", body_get["__mount_options"])



# Copyright 2022 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import abc

from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
from testtools import testcase as tc

from manila_tempest_tests.common import waiters
from manila_tempest_tests.tests.api import base
from manila_tempest_tests.tests.rbac import base as rbac_base

CONF = config.CONF


class ShareRbacSharesTests(rbac_base.ShareRbacBaseTests,
                           metaclass=abc.ABCMeta):

    @classmethod
    def skip_checks(cls):
        super(ShareRbacSharesTests, cls).skip_checks()
        if cls.protocol not in CONF.share.enable_protocols:
            message = "%s tests are disabled" % cls.protocol
            raise cls.skipException(message)

    @classmethod
    def setup_clients(cls):
        super(ShareRbacSharesTests, cls).setup_clients()
        cls.persona = getattr(cls, 'os_%s' % cls.credentials[0])
        cls.client = cls.persona.share_v2.SharesV2Client()
        cls.alt_project_share_v2_client = (
            cls.os_project_alt_member.share_v2.SharesV2Client())

    @classmethod
    def resource_setup(cls):
        super(ShareRbacSharesTests, cls).resource_setup()
        cls.share_type = cls.get_share_type()

    def share(self, share_type_id, size=None):
        share = {}
        share['name'] = data_utils.rand_name('share')
        share['size'] = size or CONF.share.share_size
        share['share_type_id'] = share_type_id
        share['share_protocol'] = self.protocol
        return share

    @abc.abstractmethod
    def test_get_share(self):
        pass

    @abc.abstractmethod
    def test_list_share(self):
        pass

    @abc.abstractmethod
    def test_create_share(self):
        pass

    @abc.abstractmethod
    def test_delete_share(self):
        pass

    @abc.abstractmethod
    def test_force_delete_share(self):
        pass

    @abc.abstractmethod
    def test_update_share(self):
        pass

    @abc.abstractmethod
    def test_reset_share(self):
        pass

    @abc.abstractmethod
    def test_shrink_share(self):
        pass

    @abc.abstractmethod
    def test_extend_share(self):
        pass

    @abc.abstractmethod
    def test_set_share_metadata(self):
        pass

    @abc.abstractmethod
    def test_get_share_metadata(self):
        pass

    @abc.abstractmethod
    def test_delete_share_metadata(self):
        pass


class TestProjectAdminTestsNFS(ShareRbacSharesTests, base.BaseSharesTest):

    credentials = ['project_admin', 'project_alt_member']
    protocol = 'nfs'

    @classmethod
    def setup_clients(cls):
        super(TestProjectAdminTestsNFS, cls).setup_clients()
        project_member = cls.setup_user_client(
            cls.persona, project_id=cls.persona.credentials.project_id)
        cls.share_member_client = project_member.share_v2.SharesV2Client()

    @decorators.idempotent_id('14a52454-cba0-4973-926a-28e924ae2e63')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_get_share(self):
        share = self.create_share(
            self.share_member_client, self.share_type['id'])
        self.do_request(
            'get_share', expected_status=200, share_id=share['id'])

        alt_share = self.create_share(
            self.alt_project_share_v2_client, self.share_type['id'])
        self.do_request(
            'get_share', expected_status=200,
            share_id=alt_share['id'])

    @decorators.idempotent_id('5f8c06e6-5b80-45f8-aefb-1b55617d1bd1')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_list_share(self):
        share = self.create_share(
            self.share_member_client, self.share_type['id'])
        alt_share = self.create_share(
            self.alt_project_share_v2_client, self.share_type['id'])

        params = {"all_tenants": 1}
        share_list = self.do_request(
            'list_shares', expected_status=200, params=params)['shares']
        share_id_list = [
            s['id'] for s in share_list
        ]

        self.assertIn(share['id'], share_id_list)
        self.assertIn(alt_share['id'], share_id_list)

    @decorators.idempotent_id('34b84af3-a9ea-4c19-8414-e4e44648099c')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_create_share(self):
        share = self.do_request(
            'create_share', expected_status=200,
            **self.share(self.share_type['id']))['share']
        waiters.wait_for_resource_status(self.client,
                                         share['id'], 'available')
        self.addCleanup(self.delete_resource, self.client,
                        share_id=share['id'])

    @decorators.idempotent_id('44f2eae6-44d4-4962-a94a-d2717b74728f')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_delete_share(self):
        share = self.create_share(
            self.share_member_client, self.share_type['id'])
        self.do_request(
            'delete_share', expected_status=202, share_id=share['id'])
        self.client.wait_for_resource_deletion(share_id=share['id'])

        alt_share = self.create_share(
            self.alt_project_share_v2_client, self.share_type['id'])
        self.do_request(
            'delete_share', expected_status=202,
            share_id=alt_share['id'])
        self.client.wait_for_resource_deletion(share_id=alt_share['id'])

    @decorators.idempotent_id('2e915a27-488d-4e33-b2f8-37758ef11653')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_force_delete_share(self):
        share = self.create_share(
            self.share_member_client, self.share_type['id'])
        self.do_request(
            'force_delete', expected_status=202, s_id=share['id'])
        self.client.wait_for_resource_deletion(share_id=share['id'])

        alt_share = self.create_share(
            self.alt_project_share_v2_client, self.share_type['id'])
        self.do_request(
            'force_delete', expected_status=202,
            s_id=alt_share['id'])
        self.client.wait_for_resource_deletion(share_id=alt_share['id'])

    @decorators.idempotent_id('5c2bda4c-0179-4af9-b18c-430a7d31f962')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_update_share(self):
        share = self.create_share(
            self.share_member_client, self.share_type['id'])
        name = data_utils.rand_name("updated_share")
        self.do_request(
            'update_share', expected_status=200,
            share_id=share['id'], name=name)

        alt_share = self.create_share(
            self.alt_project_share_v2_client, self.share_type['id'])
        name = data_utils.rand_name("updated_share")
        self.do_request(
            'update_share', expected_status=200,
            share_id=alt_share['id'], name=name)

    @decorators.idempotent_id('44fb7049-8fc0-4584-9ff1-7527395d2ec5')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_reset_share(self):
        share = self.create_share(
            self.share_member_client, self.share_type['id'])
        self.do_request(
            'reset_state', expected_status=202, s_id=share['id'],
            status="error")

        alt_share = self.create_share(
            self.alt_project_share_v2_client, self.share_type['id'])
        self.do_request(
            'reset_state', expected_status=202,
            s_id=alt_share['id'], status="error")

    @decorators.idempotent_id('cc49ae58-6696-4030-a029-a66bae2efa96')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_shrink_share(self):
        share = self.create_share(
            self.share_member_client, self.share_type['id'],
            size=CONF.share.share_size + 1)
        self.do_request(
            'shrink_share', expected_status=202, share_id=share['id'],
            new_size=CONF.share.share_size)
        waiters.wait_for_resource_status(self.client, share['id'], 'available')

        alt_share = self.create_share(
            self.alt_project_share_v2_client, self.share_type['id'],
            size=CONF.share.share_size + 1)
        self.do_request(
            'shrink_share', expected_status=202,
            share_id=alt_share['id'], new_size=CONF.share.share_size)
        waiters.wait_for_resource_status(
            self.alt_project_share_v2_client, alt_share['id'], 'available')

    @decorators.idempotent_id('2cfa04e5-16cc-43e4-b892-c1a11b0a2f2d')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_extend_share(self):
        share = self.create_share(
            self.share_member_client, self.share_type['id'])
        self.do_request(
            'extend_share', expected_status=202, share_id=share['id'],
            new_size=CONF.share.share_size + 1)
        waiters.wait_for_resource_status(self.client, share['id'], 'available')

        alt_share = self.create_share(
            self.alt_project_share_v2_client, self.share_type['id'])
        self.do_request(
            'extend_share', expected_status=202,
            share_id=alt_share['id'], new_size=CONF.share.share_size + 1)
        waiters.wait_for_resource_status(
            self.alt_project_share_v2_client, alt_share['id'], 'available')

    @decorators.idempotent_id('d6014579-d772-441a-a9b1-01b1e87caeaa')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_set_share_metadata(self):
        share = self.create_share(
            self.share_member_client, self.share_type['id'])
        self.do_request(
            'set_metadata', expected_status=200, resource_id=share['id'],
            metadata={'key': 'value'})

        alt_share = self.create_share(
            self.alt_project_share_v2_client, self.share_type['id'])
        self.do_request(
            'set_metadata', expected_status=200,
            resource_id=alt_share['id'], metadata={'key': 'value'})

    @decorators.idempotent_id('2d91e97e-d0e5-4112-8b22-60cd4659586c')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_get_share_metadata(self):
        metadata = {'key': 'value'}
        share = self.create_share(
            self.share_member_client, self.share_type['id'],
            metadata=metadata)
        self.do_request(
            'get_metadata', expected_status=200, resource_id=share['id'])

        alt_share = self.create_share(
            self.alt_project_share_v2_client, self.share_type['id'],
            metadata=metadata)
        self.do_request(
            'get_metadata', expected_status=200,
            resource_id=alt_share['id'])

    @decorators.idempotent_id('4cd807d6-bac4-4d0f-a207-c84dfe77f032')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_delete_share_metadata(self):
        share = self.create_share(
            self.share_member_client, self.share_type['id'],
            metadata={'key': 'value'})
        self.do_request(
            'delete_metadata', expected_status=200, resource_id=share['id'],
            key='key')

        alt_share = self.create_share(
            self.alt_project_share_v2_client, self.share_type['id'],
            metadata={'key': 'value'})
        self.do_request(
            'delete_metadata', expected_status=200,
            resource_id=alt_share['id'], key='key')


class TestProjectMemberTestsNFS(ShareRbacSharesTests, base.BaseSharesTest):

    credentials = ['project_member', 'project_alt_member']
    protocol = 'nfs'

    @decorators.idempotent_id('75b9fd40-ae63-4caf-9c93-0fe24b2ce904')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_get_share(self):
        share_client = getattr(self, 'share_member_client', self.client)
        share = self.create_share(share_client, self.share_type['id'])
        self.do_request(
            'get_share', expected_status=200, share_id=share['id'])

        alt_share = self.create_share(
            self.alt_project_share_v2_client, self.share_type['id'])
        self.do_request(
            'get_share', expected_status=lib_exc.NotFound,
            share_id=alt_share['id'])

    @decorators.idempotent_id('92fd157a-f357-4a08-9fc6-9e77a55b89a8')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_list_share(self):
        share_client = getattr(self, 'share_member_client', self.client)
        share = self.create_share(share_client, self.share_type['id'])
        alt_share = self.create_share(
            self.alt_project_share_v2_client, self.share_type['id'])

        # We expect this key to be ignored since project_member isn't an admin
        params = {"all_tenants": 1}
        share_list = self.do_request(
            'list_shares', expected_status=200, params=params)['shares']
        share_id_list = [
            s['id'] for s in share_list
        ]

        self.assertIn(share['id'], share_id_list)
        self.assertNotIn(alt_share['id'], share_id_list)

    @decorators.idempotent_id('7a6eef6b-bf8e-4cb3-a39c-6dc7fbe115ab')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_create_share(self):
        share = self.do_request(
            'create_share', expected_status=200,
            **self.share(self.share_type['id']))['share']
        waiters.wait_for_resource_status(self.client,
                                         share['id'], 'available')
        self.addCleanup(self.delete_resource, self.client,
                        share_id=share['id'])

    @decorators.idempotent_id('6c546ed7-ebfd-4ac5-a626-d333a25a9e66')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_delete_share(self):
        share = self.create_share(self.client, self.share_type['id'])
        self.do_request(
            'delete_share', expected_status=202, share_id=share['id'])

        alt_share = self.create_share(
            self.alt_project_share_v2_client, self.share_type['id'])
        self.do_request(
            'delete_share', expected_status=lib_exc.NotFound,
            share_id=alt_share['id'])

    @decorators.idempotent_id('2349d2b0-6314-4018-85e5-696f8d1ca94a')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_force_delete_share(self):
        share_client = getattr(self, 'share_member_client', self.client)
        share = self.create_share(share_client, self.share_type['id'])
        self.do_request(
            'force_delete', expected_status=lib_exc.Forbidden,
            s_id=share['id'])

        alt_share = self.create_share(
            self.alt_project_share_v2_client, self.share_type['id'])
        self.do_request(
            'force_delete', expected_status=lib_exc.Forbidden,
            s_id=alt_share['id'])

    @decorators.idempotent_id('20d6360d-5cea-4305-be36-7e1429007598')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_update_share(self):
        share = self.create_share(self.client, self.share_type['id'])
        name = data_utils.rand_name("rename_share")
        self.do_request(
            'update_share', expected_status=200, share_id=share['id'],
            name=name)

        alt_share = self.create_share(
            self.alt_project_share_v2_client, self.share_type['id'])
        alt_name = data_utils.rand_name("rename_share")
        self.do_request(
            'update_share', expected_status=lib_exc.NotFound,
            share_id=alt_share['id'], name=alt_name)

    @decorators.idempotent_id('483cbaef-a53d-433a-9259-f2ecc209f405')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_reset_share(self):
        share_client = getattr(self, 'share_member_client', self.client)
        share = self.create_share(share_client, self.share_type['id'])
        self.do_request(
            'reset_state', expected_status=lib_exc.Forbidden,
            s_id=share['id'], status="error")

        alt_share = self.create_share(
            self.alt_project_share_v2_client, self.share_type['id'])
        self.do_request(
            'reset_state', expected_status=lib_exc.Forbidden,
            s_id=alt_share['id'], status="error")

    @decorators.idempotent_id('56a07567-d0a9-460a-9267-fcd82306a371')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_shrink_share(self):
        share = self.create_share(self.client, self.share_type['id'], size=2)
        self.do_request(
            'shrink_share', expected_status=202,
            share_id=share['id'], new_size=CONF.share.share_size)
        waiters.wait_for_resource_status(self.client, share['id'], 'available')

        alt_share = self.create_share(
            self.alt_project_share_v2_client, self.share_type['id'], size=2)
        self.do_request(
            'shrink_share', expected_status=lib_exc.NotFound,
            share_id=alt_share['id'], new_size=CONF.share.share_size)

    @decorators.idempotent_id('c09e6a72-5b99-4be6-8ffe-8ecaad0be990')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_extend_share(self):
        share = self.create_share(self.client, self.share_type['id'])
        self.do_request(
            'extend_share', expected_status=202,
            share_id=share['id'], new_size=CONF.share.share_size + 1)
        waiters.wait_for_resource_status(self.client, share['id'], 'available')

        alt_share = self.create_share(
            self.alt_project_share_v2_client, self.share_type['id'])
        self.do_request(
            'extend_share', expected_status=lib_exc.NotFound,
            share_id=alt_share['id'], new_size=CONF.share.share_size + 1)

    @decorators.idempotent_id('f1c03630-987c-4f19-938d-4a0ef6529177')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_set_share_metadata(self):
        share = self.create_share(
            self.client, self.share_type['id'])
        self.do_request(
            'set_metadata', expected_status=200, resource_id=share['id'],
            metadata={'key': 'value'})

        alt_share = self.create_share(
            self.alt_project_share_v2_client, self.share_type['id'])
        self.do_request(
            'set_metadata', expected_status=lib_exc.NotFound,
            resource_id=alt_share['id'], metadata={'key': 'value'})

    @decorators.idempotent_id('a69a2b85-3374-4621-83a9-89937ddb520b')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_get_share_metadata(self):
        metadata = {'key': 'value'}
        share_client = getattr(self, 'share_member_client', self.client)
        share = self.create_share(share_client, self.share_type['id'],
                                  metadata=metadata)
        self.do_request(
            'get_metadata', expected_status=200, resource_id=share['id'])

        alt_share = self.create_share(
            self.alt_project_share_v2_client, self.share_type['id'],
            metadata=metadata)
        self.do_request(
            'get_metadata', expected_status=lib_exc.NotFound,
            resource_id=alt_share['id'])

    @decorators.idempotent_id('bea5518a-338e-494d-9034-1d03658ed58b')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_delete_share_metadata(self):
        share = self.create_share(
            self.client, self.share_type['id'], metadata={'key': 'value'})
        self.do_request(
            'delete_metadata', expected_status=200, resource_id=share['id'],
            key='key')

        alt_share = self.create_share(
            self.alt_project_share_v2_client, self.share_type['id'],
            metadata={'key': 'value'})
        self.do_request(
            'delete_metadata', expected_status=lib_exc.NotFound,
            resource_id=alt_share['id'], key='key')


class TestProjectReaderTestsNFS(TestProjectMemberTestsNFS):
    """Test suite for basic share operations by reader user

    In order to test certain share operations we must create a share resource
    for this. Since reader user is limited in resources creation, we are forced
    to use admin credentials, so we can test other share operations.
    In this class we use admin user to create a member user within reader
    project. That way we can perform a reader actions on this resource.
    """

    credentials = ['project_reader', 'project_admin', 'project_alt_member']

    @classmethod
    def setup_clients(cls):
        super(TestProjectReaderTestsNFS, cls).setup_clients()
        project_member = cls.setup_user_client(
            cls.os_project_admin,
            project_id=cls.persona.credentials.project_id)
        cls.share_member_client = project_member.share_v2.SharesV2Client()

    @decorators.idempotent_id('dc439eaf-c885-4002-be8f-4c488beeca81')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_get_share(self):
        super(TestProjectReaderTestsNFS, self).test_get_share()

    @decorators.idempotent_id('1fbb1078-4386-4b52-aa88-e6be4a286791')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_list_share(self):
        super(TestProjectReaderTestsNFS, self).test_list_share()

    @decorators.idempotent_id('350ba4c9-def9-4865-824a-de1ddff5dcf9')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_create_share(self):
        self.do_request(
            'create_share', expected_status=lib_exc.Forbidden,
            **self.share(self.share_type['id']))

    @decorators.idempotent_id('eb92b142-fd8d-47e3-99fe-944cce747ad7')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_delete_share(self):
        share = self.create_share(
            self.share_member_client, self.share_type['id'])
        self.do_request(
            'delete_share', expected_status=lib_exc.Forbidden,
            share_id=share['id'])

        alt_share = self.create_share(
            self.alt_project_share_v2_client, self.share_type['id'])
        self.do_request(
            'delete_share', expected_status=lib_exc.Forbidden,
            share_id=alt_share['id'])

    @decorators.idempotent_id('cb040955-5897-409f-aea0-84b6ae16b77e')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_force_delete_share(self):
        super(TestProjectReaderTestsNFS, self).test_force_delete_share()

    @decorators.idempotent_id('3184269a-11ca-4484-8a4d-b855a6e1800f')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_update_share(self):
        share = self.create_share(
            self.share_member_client, self.share_type['id'])
        name = data_utils.rand_name("rename_share")
        self.do_request(
            'update_share', expected_status=lib_exc.Forbidden,
            share_id=share['id'], name=name)

        alt_share = self.create_share(
            self.alt_project_share_v2_client, self.share_type['id'])
        alt_name = data_utils.rand_name("rename_share")
        self.do_request(
            'update_share', expected_status=lib_exc.Forbidden,
            share_id=alt_share['id'], name=alt_name)

    @decorators.idempotent_id('e5ae5b56-38c0-44ec-b8e0-4bc2a5c1d28a')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_reset_share(self):
        super(TestProjectReaderTestsNFS, self).test_reset_share()

    @decorators.idempotent_id('f85818b1-b93a-4b89-8aa4-b099e582be7c')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_shrink_share(self):
        share = self.create_share(
            self.share_member_client, self.share_type['id'],
            size=CONF.share.share_size + 1)
        self.do_request(
            'shrink_share', expected_status=lib_exc.Forbidden,
            share_id=share['id'], new_size=CONF.share.share_size)
        waiters.wait_for_resource_status(self.client, share['id'], 'available')

        alt_share = self.create_share(
            self.alt_project_share_v2_client, self.share_type['id'],
            size=CONF.share.share_size + 1)
        self.do_request(
            'shrink_share', expected_status=lib_exc.Forbidden,
            share_id=alt_share['id'], new_size=CONF.share.share_size)

    @decorators.idempotent_id('0b57aedb-6b68-498f-814e-173c47e6c307')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_extend_share(self):
        share = self.create_share(
            self.share_member_client, self.share_type['id'])
        self.do_request(
            'extend_share', expected_status=lib_exc.Forbidden,
            share_id=share['id'], new_size=CONF.share.share_size + 1)
        waiters.wait_for_resource_status(self.client, share['id'], 'available')

        alt_share = self.create_share(
            self.alt_project_share_v2_client, self.share_type['id'])
        self.do_request(
            'extend_share', expected_status=lib_exc.Forbidden,
            share_id=alt_share['id'], new_size=CONF.share.share_size + 1)

    @decorators.idempotent_id('3def3f4e-33fc-4726-8818-6cffbc2cab51')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_set_share_metadata(self):
        share = self.create_share(
            self.share_member_client, self.share_type['id'])
        self.do_request(
            'set_metadata', expected_status=lib_exc.Forbidden,
            resource_id=share['id'], metadata={'key': 'value'})

        alt_share = self.create_share(
            self.alt_project_share_v2_client, self.share_type['id'])
        self.do_request(
            'set_metadata', expected_status=lib_exc.Forbidden,
            resource_id=alt_share['id'], metadata={'key': 'value'})

    @decorators.idempotent_id('28cacc77-556f-4707-ba2b-5ef3e56d6ef9')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_get_share_metadata(self):
        super(TestProjectReaderTestsNFS, self).test_get_share_metadata()

    @decorators.idempotent_id('55486589-a4ef-44f2-b489-96bc29dcd243')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_delete_share_metadata(self):
        share = self.create_share(
            self.share_member_client, self.share_type['id'],
            metadata={'key': 'value'})
        self.do_request(
            'delete_metadata', expected_status=lib_exc.Forbidden,
            resource_id=share['id'], key='key')

        alt_share = self.create_share(
            self.alt_project_share_v2_client, self.share_type['id'],
            metadata={'key': 'value'})
        self.do_request(
            'delete_metadata', expected_status=lib_exc.Forbidden,
            resource_id=alt_share['id'], key='key')


class TestProjectAdminTestsCEPHFS(TestProjectAdminTestsNFS):
    protocol = 'cephfs'


class TestProjectMemberTestsCEPHFS(TestProjectMemberTestsNFS):
    protocol = 'cephfs'


class TestProjectReaderTestsCEPHFS(TestProjectReaderTestsNFS):
    protocol = 'cephfs'


class TestProjectAdminTestsCIFS(TestProjectAdminTestsNFS):
    protocol = 'cifs'


class TestProjectMemberTestsCIFS(TestProjectMemberTestsNFS):
    protocol = 'cifs'


class TestProjectReaderTestsCIFS(TestProjectReaderTestsNFS):
    protocol = 'cifs'



# Copyright 2019 NetApp Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ddt
from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
import testtools
from testtools import testcase as tc

from manila_tempest_tests.common import constants
from manila_tempest_tests.common import waiters
from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils


CONF = config.CONF
LATEST_MICROVERSION = CONF.share.max_api_microversion


@ddt.ddt
class ShareNetworkSubnetsNegativeTest(base.BaseSharesAdminTest):

    @classmethod
    def skip_checks(cls):
        super(ShareNetworkSubnetsNegativeTest, cls).skip_checks()
        utils.check_skip_if_microversion_not_supported("2.51")

    @classmethod
    def resource_setup(cls):
        super(ShareNetworkSubnetsNegativeTest, cls).resource_setup()
        # Create a new share network which will be used in the tests
        cls.share_network = cls.shares_v2_client.create_share_network(
            cleanup_in_class=True)['share_network']
        cls.share_network_id = cls.share_network['id']
        cls.share_type = cls.create_share_type()
        cls.az = (
            cls.shares_v2_client.list_availability_zones()
            ['availability_zones'][0]
        )
        cls.az_name = cls.az['name']

    @decorators.idempotent_id('d20b6105-22d1-4fc0-8468-45dd019240c0')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_add_share_network_subnet_share_network_not_found(self):
        data = utils.generate_subnet_data()
        self.assertRaises(lib_exc.NotFound,
                          self.shares_v2_client.create_subnet,
                          'fake_inexistent_id',
                          **data)

    @decorators.idempotent_id('68264ac2-57e5-47ea-aa71-198c020be975')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_add_share_network_subnet_az_not_found(self):
        data = {'availability_zone': 'non-existent-az'}

        self.assertRaises(lib_exc.BadRequest,
                          self.shares_v2_client.create_subnet,
                          self.share_network_id, **data)

    @decorators.idempotent_id('13f397bf-5e3a-42b0-b4f9-9cd2dbbb0955')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    @ddt.data(True, False)
    @utils.skip_if_is_microversion_ge(LATEST_MICROVERSION, "2.70")
    def test_add_share_network_subnet_in_same_az_exists(self, is_default):
        share_network = self.shares_v2_client.create_share_network(
            )['share_network']
        data = {}

        if not is_default:
            azs = self.get_availability_zones_matching_share_type(
                self.share_type)
            data['availability_zone'] = azs[0]
            self.shares_v2_client.create_subnet(
                share_network['id'], **data)

        self.assertRaises(lib_exc.Conflict,
                          self.shares_v2_client.create_subnet,
                          share_network['id'], **data)

    @decorators.idempotent_id('36e991c1-f09b-47dc-adc9-3da388b14612')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_add_share_network_subnet_missing_parameters(self):
        # Generate subnet data
        data = utils.generate_subnet_data()
        data['availability_zone'] = self.az_name

        data.pop('neutron_net_id')
        self.assertRaises(lib_exc.BadRequest,
                          self.shares_v2_client.create_subnet,
                          self.share_network_id, **data)

    @decorators.idempotent_id('11e76b07-f3e3-401f-9469-256b93a768e6')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_show_subnet_share_network_not_found(self):
        self.assertRaises(lib_exc.NotFound,
                          self.shares_v2_client.get_subnet,
                          'fake-subnet',
                          'fake-sn')

    @decorators.idempotent_id('22d4528b-39ae-4f97-b1b2-1b9d65146c97')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_show_subnet_not_found(self):
        self.assertRaises(lib_exc.NotFound,
                          self.shares_v2_client.get_subnet,
                          'fake-subnet',
                          self.share_network_id)

    @decorators.idempotent_id('9dfc6ba3-c7be-4a6d-b104-ce77d2337bb3')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_get_deleted_subnet(self):
        # Generate subnet data
        data = utils.generate_subnet_data()
        data['share_network_id'] = self.share_network_id
        az = self.shares_v2_client.list_availability_zones(
            )['availability_zones'][0]
        data['availability_zone'] = az['name']

        subnet = self.create_share_network_subnet(**data)

        # Make sure that the created subnet contains the data
        self.assertLessEqual(data.items(), subnet.items())

        # Delete the given subnet
        self.shares_v2_client.delete_subnet(self.share_network_id,
                                            subnet['id'])
        share_network = self.shares_v2_client.get_share_network(
            self.share_network_id
        )['share_network']

        self.assertIsNotNone(share_network)
        self.assertRaises(lib_exc.NotFound,
                          self.shares_v2_client.get_subnet,
                          subnet['id'],
                          self.share_network['id'])

    @decorators.idempotent_id('bf9d63c4-37fb-4646-add7-905b196316f1')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @testtools.skipIf(not CONF.share.multitenancy_enabled,
                      'Can run only with drivers that do handle share servers '
                      'creation. Skipping.')
    @testtools.skipIf(not CONF.share.run_manage_unmanage_tests,
                      'Can run only with manage/unmanage tests enabled.')
    def test_delete_contains_unmanaged_share_servers(self):
        # Get a compatible availability zone
        az = self.get_availability_zones_matching_share_type(
            self.share_type)[0]

        share_network = self.shares_v2_client.get_share_network(
            self.shares_v2_client.share_network_id
        )['share_network']
        share_network_id = share_network['id']
        subnet = utils.share_network_get_default_subnet(share_network)

        # Generate subnet data
        data = {'neutron_net_id': subnet['neutron_net_id'],
                'neutron_subnet_id': subnet['neutron_subnet_id'],
                'share_network_id': share_network_id,
                'availability_zone': az}

        # Create a new subnet in the desired az
        subnet = self.create_share_network_subnet(**data)

        args = {'share_network_id': share_network_id,
                'share_type_id': self.share_type['id'],
                'availability_zone': az}

        # Create a share into the share network
        share = self.shares_v2_client.create_share(**args)['share']
        waiters.wait_for_resource_status(
            self.shares_v2_client, share['id'], constants.STATUS_AVAILABLE)
        share = self.shares_v2_client.get_share(share['id'])['share']

        # Gets the export locations to be used in the future
        el = self.shares_v2_client.list_share_export_locations(
            share['id'])['export_locations']
        share['export_locations'] = el

        # Unmanages the share to make the share server become is_auto
        # deletable=False
        self._unmanage_share_and_wait(share)

        # Assert that the user cannot delete a subnet that contains share
        # servers which may have unmanaged stuff
        self.assertRaises(lib_exc.Conflict,
                          self.shares_v2_client.delete_subnet,
                          share_network_id,
                          subnet['id'])

        # Manages the share again to start cleaning up the test stuff
        managed_share = self.shares_v2_client.manage_share(
            service_host=share['host'],
            export_path=share['export_locations'][0],
            protocol=share['share_proto'],
            share_type_id=self.share_type['id'],
            name='share_to_be_deleted',
            description='share managed to be deleted',
            share_server_id=share['share_server_id']
        )['share']

        # Do some necessary cleanup
        waiters.wait_for_resource_status(
            self.shares_v2_client, managed_share['id'],
            constants.STATUS_AVAILABLE)
        self.shares_client.delete_share(managed_share['id'])
        self.shares_v2_client.wait_for_resource_deletion(
            share_id=managed_share["id"])
        self._delete_share_server_and_wait(share['share_server_id'])
        self.shares_v2_client.delete_subnet(share_network_id,
                                            subnet['id'])

    @decorators.idempotent_id('5088edfe-2800-4dec-a982-42c864bba954')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @testtools.skipIf(not CONF.share.multitenancy_enabled,
                      'Can run only with drivers that do handle share servers '
                      'creation. Skipping.')
    @testtools.skipIf(CONF.share.share_network_id != "",
                      "This test is not suitable for pre-existing "
                      "share_network.")
    def test_delete_contains_shares(self):
        # Get a compatible availability zone
        az = self.get_availability_zones_matching_share_type(
            self.share_type)[0]
        check_multiple_subnets = utils.is_microversion_ge(
            CONF.share.max_api_microversion, '2.70')
        original_share_network = self.shares_v2_client.get_share_network(
            self.shares_v2_client.share_network_id
        )['share_network']
        share_net_info = (
            utils.share_network_get_default_subnet(original_share_network))
        share_network = self.create_share_network(
            neutron_net_id=share_net_info['neutron_net_id'],
            neutron_subnet_id=share_net_info['neutron_subnet_id'],
        )
        share_network = self.shares_v2_client.get_share_network(
            share_network['id']
        )['share_network']
        share_network_id = share_network['id']
        default_subnet = share_network['share_network_subnets'][0]

        # Generate subnet data
        data = {'neutron_net_id': default_subnet['neutron_net_id'],
                'neutron_subnet_id': default_subnet['neutron_subnet_id'],
                'share_network_id': share_network_id,
                'availability_zone': az}

        # Create a new subnet in the desired az
        subnet = self.create_share_network_subnet(**data)

        args = {'share_network_id': share_network_id,
                'share_type_id': self.share_type['id'],
                'availability_zone': az}

        # Create a share into the share network
        share = self.shares_v2_client.create_share(**args)['share']
        waiters.wait_for_resource_status(
            self.shares_v2_client, share['id'], constants.STATUS_AVAILABLE)
        share = self.admin_shares_v2_client.get_share(share['id'])['share']
        share_server = self.admin_shares_v2_client.show_share_server(
            share['share_server_id']
        )['share_server']
        # Match share server subnet
        if check_multiple_subnets:
            self.assertIn(subnet['id'],
                          share_server['share_network_subnet_ids'])
        else:
            self.assertEqual(subnet['id'],
                             share_server['share_network_subnet_id'])

        # Assert that the user cannot delete a subnet that contain shares
        self.assertRaises(lib_exc.Conflict,
                          self.shares_v2_client.delete_subnet,
                          share_network_id,
                          subnet['id'])
        # Assert that the user cannot delete a share-network that contain
        # shares
        self.assertRaises(lib_exc.Conflict,
                          self.shares_v2_client.delete_share_network,
                          share_network_id)
        # Cleanups
        self.shares_client.delete_share(share['id'])
        self.shares_v2_client.wait_for_resource_deletion(share_id=share["id"])
        self._delete_share_server_and_wait(share['share_server_id'])
        self.shares_v2_client.delete_subnet(share_network_id,
                                            subnet['id'])

    @decorators.idempotent_id('1025b69b-f1fd-4e44-b749-33779545ffe8')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_delete_subnet_share_network_not_found(self):
        self.assertRaises(lib_exc.NotFound,
                          self.shares_v2_client.delete_subnet,
                          'fake-sn',
                          'fake-subnet')

    @decorators.idempotent_id('a5bdc817-0c34-4856-9732-f561e6ab5a83')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_delete_subnet_not_found(self):
        self.assertRaises(lib_exc.NotFound,
                          self.shares_v2_client.delete_subnet,
                          self.share_network_id,
                          'fake-subnet')
